package com.gx.obe.opening.composite;

import java.util.Properties;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.utils.WidgetUtils;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.gx.obe.VideoConfigConstants;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.common.widget.shell.ToolsShell;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.ObjectUtils;
import com.swtdesigner.ResourceManager;

public class ComfigLivingAddressComposite extends Composite {
	private Properties videoProperties;
	private Text livingAddessText;
	private Composite baiduLivingTypeComposite;
	private Text baiduAccountNameText;
	private Button btnRtmp;
	private Button btnHttp;
	private Button otherLivingBtn;
	private Button baiduLivingBtn;
	private Composite composite;
	private Label baiduAccountName;
	private ToolsShell.ClickListener clickListener;
	
	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */
	public ComfigLivingAddressComposite(Composite parent, int style) {
		super(parent, style);
		setLayout(new GridLayout(1, false));
		
		composite = new Composite(this, SWT.NONE);
		composite.setLayout(new GridLayout(4, false));
		composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		baiduLivingBtn = new Button(composite, SWT.RADIO);
		baiduLivingBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				selectLivingTypeAction();
			}
		});
		baiduLivingBtn.setText(Messages.Config_baiduLiveBroadcast);
		
		baiduAccountName = new Label(composite, SWT.NONE);
		baiduAccountName.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		baiduAccountName.setText(Messages.Config_liveStreamName + "：");

		baiduAccountNameText = new Text(composite, SWT.BORDER);
		GridData gd_livingNametext = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
		gd_livingNametext.widthHint = 200;
		baiduAccountNameText.setLayoutData(gd_livingNametext);
		
		baiduLivingTypeComposite = new Composite(composite, SWT.NONE);
		baiduLivingTypeComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		baiduLivingTypeComposite.setLayout(new GridLayout(3, false));
		
		btnRtmp = new Button(baiduLivingTypeComposite, SWT.RADIO);
		btnRtmp.setText("RTMP");
		btnRtmp.setSelection(true);
		btnRtmp.setData("livingType", CommonEnum.LIVING_COM_RTMP);
		btnHttp = new Button(baiduLivingTypeComposite, SWT.RADIO);
		btnHttp.setText("HTTP");
		btnHttp.setData("livingType", CommonEnum.LIVING_COM_HTTP);
		
		otherLivingBtn = new Button(composite, SWT.RADIO);
		otherLivingBtn.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 2, 1));
		otherLivingBtn.setText(Messages.Config_otherLiveAddress + "：");
		otherLivingBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e){
				selectLivingTypeAction();
			}
		});
		livingAddessText = new Text(composite, SWT.BORDER);
		livingAddessText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
		
		Composite buttonComposite = new Composite(this, SWT.NONE);
		buttonComposite.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, false, 1, 1));
		buttonComposite.setLayout(new GridLayout(2, true));

		EButton saveLivingAddress = new EButton(buttonComposite, SWT.NONE | EButton.CSS_BLUE);
		saveLivingAddress.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/white/save.png"));
		GridData gd_saveLivingAddress = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
		saveLivingAddress.setLayoutData(gd_saveLivingAddress);
		saveLivingAddress.setText(Messages.Button_save);
		saveLivingAddress.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				saveLivingAddress();
			}
		});
		
		EButton copyLivingAddress = new EButton(buttonComposite, SWT.NONE | EButton.CSS_BLUE);
		copyLivingAddress.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/white/copy.png"));
		GridData gd_copyLivingAddress = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
		copyLivingAddress.setLayoutData(gd_copyLivingAddress);
		copyLivingAddress.setText(Messages.Config_copyLiveAddress);
		copyLivingAddress.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				copyLivingAddress();
			}
		});
		init();
	}

	/**
	 * 初始化
	 */
	private void init(){
		livingAddessText.setEnabled(false);
		baiduAccountNameText.setEnabled(false);
		videoProperties = PropertiesUtils.getProperties("videoConfig.properties");
		String livingType = videoProperties.getProperty(VideoConfigConstants.LIVING_TYPE, CommonEnum.LIVING_COM_OTHER);
		if(null != livingType && livingType.equals(CommonEnum.LIVING_COM_OTHER)){
			otherLivingBtn.setSelection(true);
			baiduLivingBtn.setSelection(false);
		}else {
			otherLivingBtn.setSelection(false);
			baiduLivingBtn.setSelection(true);
		}
		WidgetUtils.setRadioChecked(baiduLivingTypeComposite, livingType, "livingType");
		livingAddessText.setText(ObjectUtils.getObjString(videoProperties.getProperty(VideoConfigConstants.VIDEO_LIVE_URL)));
		baiduAccountNameText.setText(ObjectUtils.getObjString(videoProperties.getProperty(VideoConfigConstants.LIVING_NAME)));
		selectLivingTypeAction();
	}
	
	/**
	 *设置按钮与文本的可使用状态
	 * @param type
	 */
	private void selectLivingTypeAction(){
		if(otherLivingBtn.getSelection()){
			livingAddessText.setEnabled(true);
			livingAddessText.setText(ObjectUtils.getObjString(videoProperties.getProperty(VideoConfigConstants.VIDEO_LIVE_URL)));
			baiduAccountNameText.setEnabled(false);
			baiduLivingTypeComposite.setEnabled(false);
		}else{
			baiduAccountNameText.setEnabled(true);
			baiduLivingTypeComposite.setEnabled(true);
			baiduAccountNameText.setText(ObjectUtils.getObjString(videoProperties.getProperty(VideoConfigConstants.LIVING_NAME)));
			livingAddessText.setText("");
			livingAddessText.setEnabled(false);
			String livingType = ObjectUtils.getObjString(WidgetUtils.getRadioVal(baiduLivingTypeComposite, "livingType"));
			if(null == livingType || livingType.length() == 0){
				livingType = CommonEnum.LIVING_COM_HTTP;
				WidgetUtils.setRadioChecked(baiduLivingTypeComposite, livingType, "livingType");
			}
		}
	}
	
	/**
	 * 保存直播地址
	 * @param livingAddress
	 */
	private void copyLivingAddress(){
		String livingAddress = VideoConfigConstants.getLivingAddress();
		if(null != livingAddress && livingAddress.length() > 0){
			Clipboard clipboard = new Clipboard(Display.getDefault()); 
			TextTransfer textTransfer = TextTransfer.getInstance(); 
			clipboard.setContents(new String[]{VideoConfigConstants.getLivingAddress()}, new Transfer[]{textTransfer}); 
			clipboard.dispose(); 
			MessageDialog.openShortMessage("已复制到剪贴板！");
		}else {
			MessageDialog.openShortMessage("请先配置并保存后再复制直播地址！");
		}
	}
	
	/**
	 * 保存直播地址
	 * @param livingAddress
	 */
	private void saveLivingAddress(){
		if(null != videoProperties){
			String livingType = CommonEnum.LIVING_COM_OTHER;
			if(baiduLivingBtn.getSelection()){
				if(baiduAccountNameText.getText().trim().length() == 0){
					MessageDialog.openShortMessage("请设置直播流名称！");
					baiduAccountNameText.forceFocus();
					return;
				}
				livingType = ObjectUtils.getObjString(WidgetUtils.getRadioVal(baiduLivingTypeComposite, "livingType"));
			}else{
				if(livingAddessText.getText().trim().length() == 0){
					MessageDialog.openShortMessage("请设置直播地址！");
					livingAddessText.forceFocus();
					return;
				}
			}
			videoProperties.setProperty(VideoConfigConstants.LIVING_TYPE, livingType);
			videoProperties.setProperty(VideoConfigConstants.LIVING_NAME, baiduAccountNameText.getText().trim());
			videoProperties.setProperty(VideoConfigConstants.VIDEO_LIVE_URL, livingAddessText.getText().trim());
			PropertiesUtils.saveProperties(videoProperties);
			MessageDialog.openShortMessage(Messages.Tips_saveSuccess);
			if(null != clickListener){
				clickListener.click();
			}
		}
	}

	public void addClickListener(ToolsShell.ClickListener clickListener){
		this.clickListener = clickListener;
	}
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
}
