package com.gx.obe.opening.composite;

import java.text.MessageFormat;
import java.util.Date;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.widgets.ConfirmDialog;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;

import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.message.Messages;
import com.gx.obe.opening.shell.OpeningTipsShell;
import com.gx.obe.util.utils.DateUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.utils.NowTimeUtils;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

public class OpeningTimeComposite extends Composite{

	protected Date bidOpeningTime;
	protected Date signEndTime;
	protected Date decryptEndTime;
	protected boolean isPriceStage = false;
	public OpeningTimeComposite(Composite parent, int style) {
		super(parent, style);
		// TODO Auto-generated constructor stub
	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
	
	public StringBuilder getStartOpeningHallConfirmMsg(){
		StringBuilder message = new StringBuilder();
		message.append(MessageFormat.format(Messages.Tips_theBidOpeningTime, DateUtils.getFormat(bidOpeningTime, Messages.Format_dateMinuteCNStr)));
		if(null != signEndTime){
			message.append("\r\n");
			message.append(MessageFormat.format(Messages.Tips_theSigninEndTime, DateUtils.getFormat(signEndTime, Messages.Format_dateMinuteCNStr)));
			if(signEndTime.getTime() > bidOpeningTime.getTime()){
				message.append(MessageFormat.format(Messages.Tips_theSigninClosed, DateUtils.timeSpan(signEndTime.getTime() - bidOpeningTime.getTime())));
			}else if(signEndTime.getTime() < bidOpeningTime.getTime()){
				message.append(MessageFormat.format(Messages.Tips_theSigninClosedBefore, DateUtils.timeSpan(bidOpeningTime.getTime() - signEndTime.getTime())));
			}else {
				message.append(Messages.Tips_theSigninClosedOpengingTime);
			}
		}
		if(null != decryptEndTime){
			message.append("\r\n");
			message.append(MessageFormat.format(Messages.Tips_theDecryptEndTime, DateUtils.getFormat(decryptEndTime, Messages.Format_dateMinuteCNStr)));
			if(decryptEndTime.getTime() > bidOpeningTime.getTime()){
				message.append(MessageFormat.format(Messages.Tips_theDecryptClosed, DateUtils.timeSpan(decryptEndTime.getTime() - bidOpeningTime.getTime())));
			}
		}
		
		return message;
	}
	
	public StringBuilder getStartDecryptConfirmMsg(){
		StringBuilder message = new StringBuilder();
		message.append(MessageFormat.format(Messages.Tips_theBidOpeningTime, DateUtils.getFormat(bidOpeningTime, Messages.Format_dateMinuteCNStr)));
		if(null != decryptEndTime){
			message.append("\r\n");
			message.append(MessageFormat.format(Messages.Tips_theDecryptEndTime, DateUtils.getFormat(decryptEndTime, Messages.Format_dateMinuteCNStr)));
			message.append("\r\n");
			message.append(MessageFormat.format(Messages.Tips_theDecryptEndTime, DateUtils.getFormat(decryptEndTime, Messages.Format_dateMinuteCNStr)));
			if(decryptEndTime.getTime() > bidOpeningTime.getTime()){
				message.append(MessageFormat.format(Messages.Tips_theDecryptClosed, DateUtils.timeSpan(decryptEndTime.getTime() - bidOpeningTime.getTime())));
			}
		}
		return message;
	}
	
	/**
	 * @Description: 检查是否可以启动解密
	 * @author guoyr
	 * @return 
	 */
	public boolean checkStartOpeningHall(){
		if(null == bidOpeningTime){
			return false;
		}
		
		if (bidOpeningTime.after(NowTimeUtils.getNowDate())) {
			int startOpeningHallTime = ObjectUtils.getIntValue(PropertiesUtils.getProperty("startOpeningHallTime", "2"), 2);
			// 如果启动开标大厅时间超过开标时间超过2小时，则不允许启动
			if (bidOpeningTime.getTime() - NowTimeUtils.getNowDate().getTime() > 1000*60*60 * startOpeningHallTime) {
				StringBuilder message = new StringBuilder();
				message.append(MessageFormat.format(Messages.Tips_theBidOpeningTime, DateUtils.getFormat(bidOpeningTime, Messages.Format_dateMinuteCNStr)));
				message.append("\r\n");
				message.append(MessageFormat.format(Messages.Tips_openingRemainingTime, DateUtils.timeSpan(bidOpeningTime.getTime() - NowTimeUtils.getNowDate().getTime())));
				ConfirmDialog confirmDialog = new ConfirmDialog(getShell(), null, message.toString(), null);
				confirmDialog.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/tips_info.png"), -1, -1);
				confirmDialog.setTipsColor(SWTResourceManager.getColor(255, 0, 0));
				confirmDialog.setTipsAlignment(SWT.CENTER);
				confirmDialog.setTips(MessageFormat.format(Messages.Tips_tooSoonToOpenHall, startOpeningHallTime));
				confirmDialog.addCancel(Messages.Button_iKnown, null);
				confirmDialog.open();
				return false;
			}
		}
		return true;
	}
	/**
	 * @Description: 检查是否可以启动解密
	 * @author guoyr
	 * @return 
	 */
	public boolean checkStartDecrypt(){
		if(null == bidOpeningTime){
			return false;
		}
		
		if(bidOpeningTime.after(NowTimeUtils.getNowDate())){
			ConfirmDialog confirmDialog = new ConfirmDialog(getShell(), null, Messages.Tips_timeBeforeForBidOpening, null);
			confirmDialog.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/tips_info.png"), -1, -1);
			confirmDialog.addCancel(Messages.Button_cancel, null);
			confirmDialog.open();
			return false;
		}

		
		if(null != decryptEndTime){
			if(NowTimeUtils.getNowDate().after(decryptEndTime)){
				ConfirmDialog confirmDialog = new ConfirmDialog(getShell(), null, Messages.Tips_decryptionDeadline, null);
				confirmDialog.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/tips_info.png"), -1, -1);
				confirmDialog.addCancel(Messages.Button_cancel, null);
				confirmDialog.open();
				return false;
			}
			
			if(decryptEndTime.getTime() - bidOpeningTime.getTime() <= 1000* 60){
				StringBuilder message = new StringBuilder();
				message.append(MessageFormat.format(Messages.Tips_theBidOpeningTime, DateUtils.getFormat(bidOpeningTime, Messages.Format_dateMinuteCNStr)));
				if(decryptEndTime.getTime() > bidOpeningTime.getTime()){
					message.append("\r\n");
					message.append(MessageFormat.format(Messages.Tips_openingRemainingTime, DateUtils.timeSpan(bidOpeningTime.getTime() - NowTimeUtils.getNowDate().getTime())));
					
				}
				message.append("\r\n");
				message.append(Messages.Tips_ensureTecryptTime);
				MessageDialog.openWarning(getShell(), message.toString());
				return false;
			}
		}
		return true;
	}
	/**
	 * @Description: 是否匿名显示
	 * @author guoyr
	 * @param isAnonymous
	 * @return 
	 */
	public boolean isShowAnonymName(boolean isAnonymous) {
//		isAnonymous = true;
		if(null != bidOpeningTime && isAnonymous){
			if(bidOpeningTime.after(NowTimeUtils.getNowDate())){
				return true;
			}
		}
		return false;
		
	}
	
	/**
	 * @Description: 显示开标提示消息
	 * @author guoyr
	 * @param parentControl 
	 */
	public void showOpeningTips(Control parentControl) {
		if("01".equals(PropertiesUtils.getProperty(ConfigConstants.SHOW_OPENING_TIPS, "01"))) {
			new Thread(new Runnable() {
				public void run() {
					Display.getDefault().syncExec(new Runnable() {
						public void run() {
							if(!parentControl.isDisposed()) {
								OpeningTipsShell tipsShell = new OpeningTipsShell(getShell());
								Rectangle opRectangle = parentControl.getBounds();
								Point point = parentControl.toDisplay(SWT.DEFAULT, SWT.DEFAULT);
								tipsShell.pack();
								Point shellPoint = tipsShell.computeSize(SWT.DEFAULT, SWT.DEFAULT);
								int shellHeight = Math.max(shellPoint.y, opRectangle.height);
								if(shellPoint.x > opRectangle.width) {
									tipsShell.setSize(shellPoint.x, shellHeight);
								}else {
									tipsShell.setSize(opRectangle.width, shellHeight);
								}
								tipsShell.setLocation(point.x, point.y - shellHeight);
								tipsShell.open();
								if(shellPoint.x > opRectangle.width) {
									tipsShell.setLocation(tipsShell.getLocation().x - (shellPoint.x - opRectangle.width)/2, point.y - shellHeight);
								} 
							}
						}
					});
				}
			}).start();
		}
	
	}
}
