package com.gx.obe.opening.composite;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.message.Messages;
import com.gx.obe.opening.listener.SupplierSignListener;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.tools.ca.CACert;
import com.gx.tools.ca.CAEngineFactory;
import com.gx.tools.ole.DocumentFactory;
import com.gx.tools.ole.DocumentLoader;
import com.lowagie.text.Cell;
import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Element;
import com.lowagie.text.Font;
import com.lowagie.text.PageSize;
import com.lowagie.text.Paragraph;
import com.lowagie.text.Table;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfWriter;
import com.swtdesigner.SWTResourceManager;

/** 
 * @Description: 投标人签到结果
 * @author guoyr 
 */
public class SupplierSignResultComposite extends Composite {

	public TenderProjectEntity tenderProjectEntity;
	public Supplier supplier;
	private CACert caCert;
	private SupplierSignListener supplierSignListener;
	private Composite pdfLoadComposite;
	private File signatureFile;
	private DocumentLoader documentLoader;

	/** 
	 * @author guoyr
	 * @param parent
	 * @param style 
	 */
	public SupplierSignResultComposite(Composite parent, int style) {
		super(parent, style);
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		setLayout(gridLayout);
		
		Composite composite = new Composite(this, SWT.NONE);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		GridLayout gl_composite = new GridLayout(1, false);
		gl_composite.marginWidth = 0;
		gl_composite.marginHeight = 0;
		gl_composite.verticalSpacing = 0;
		composite.setLayout(gl_composite);
		
		pdfLoadComposite = new Composite(composite, SWT.NONE);
		pdfLoadComposite.setLayout(new FillLayout(SWT.HORIZONTAL));
		pdfLoadComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		ESepator label = new ESepator(composite, SWT.SEPARATOR | SWT.HORIZONTAL);
		label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		label.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		
		Composite composite_2 = new Composite(composite, SWT.NONE);
		GridLayout gl_composite_2 = new GridLayout(2, false);
		gl_composite_2.marginHeight = 20;
		composite_2.setLayout(gl_composite_2);
		composite_2.setLayoutData(new GridData(SWT.CENTER, SWT.FILL, true, false, 1, 1));
		
		Label label_1 = new Label(composite_2, SWT.NONE);
		label_1.setFont(SWTResourceManager.getFont("微软雅黑", 15, SWT.BOLD));
		label_1.setText("贵单位已经签到！");
		
		EButton button_cancle = new EButton(composite_2, SWT.NONE | EButton.CSS_GREEN);
		button_cancle.setFont(SWTResourceManager.getFont("微软雅黑", 12, SWT.BOLD));
		button_cancle.setText(Messages.Button_close);
		GridData gd_button_cancle = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_button_cancle.widthHint = 100;
		gd_button_cancle.heightHint = 50;
		button_cancle.setLayoutData(gd_button_cancle);
		button_cancle.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent mouseevent) {
				for(Control control : pdfLoadComposite.getChildren()){
					control.dispose();
				}
				supplierSignListener.ignoreSupplier(supplier.getSupplierName());
				supplierSignListener.gotoSupplierWelcomePage();
//				supplierSignListener.reReadCa(supplier.getSupplierName());
			}
		});

		init();
	}
	
	/**
	 * @Description: 初始化
	 * @author guoyr
	 */
	public void init(){
		
	}

	/**
	 * @Description: 刷新
	 * @author guoyr
	 * @param tenderProjectEntity
	 * @param supplier
	 */
	public void refresh(TenderProjectEntity tenderProjectEntity, Supplier supplier, CACert caCert, SupplierSignListener supplierSignListener){
		this.supplierSignListener = supplierSignListener;
		this.tenderProjectEntity = tenderProjectEntity;
		this.caCert = caCert;
		this.supplier = supplier;
		String signPdfPath = FileConstants.getSupplierSignFilePath(tenderProjectEntity.getId(), supplier.getOrgCode());
		writePDF(signPdfPath);
		showPDF(signPdfPath);
	}

	/**
	 * @Description:展示pdf签章界面，加载pdf进行签章操作 
	 * @author guoyr
	 * @param filePath
	 */
	private void showPDF(String filePath){
		for(Control control : pdfLoadComposite.getChildren()){
			control.dispose();
		}
		documentLoader = DocumentFactory.getDocumentLoader(filePath, CAEngineFactory.getSignatureTypeByCaType(caCert.getCaEngine().getCaType()));
		documentLoader.openDocument(pdfLoadComposite, true);
	}
	
	/**
	 * @Description: 写入pdf文件
	 * @author guoyr
	 * @param filePath
	 */
	private  void writePDF(String filePath) {
		signatureFile = new File(filePath);
		//判断当前本地 文件目录是否存在，不存在就创建目录。
		if(!signatureFile.getParentFile().exists()){
			signatureFile.getParentFile().mkdirs();
		}
		//判断当前 要写入的本地文件 是否存在，不存在就创建新的文件
		if(!signatureFile.exists()){
			try {
				signatureFile.createNewFile();
			} catch (IOException e) {
			}
		}
		//判断当前 投标人是否已经 签到，如果签到 了，签到 文件存在，直接加载 签到文件数据 ，如果 不存在 ，则需要写入签到数据
		if(supplier.getSignFile()!=null){
			FileOutputStream out = null;
			byte[] bytes = supplier.getSignFile();
			try {
				out = new FileOutputStream(signatureFile);
				out.write(bytes);
			} catch (Exception e) {
			}finally{
				try {
					out.flush();
					out.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
				
			}
			
		}else{
			//投标人还没签到，所以没有对应 签到文件 
			FileOutputStream out = null;
			try {
				//创建文档,设置页面大小,      左、右、上和下页边距。
				Document document = new Document(PageSize.A4, 50, 50, 50, 50);
				//document是创建的文档,out是输出
				PdfWriter writer =PdfWriter.getInstance(document, new FileOutputStream(signatureFile));
				writer.setViewerPreferences(PdfWriter.PageModeUseOutlines);
				document.open();
				//----------------
				BaseFont bfChinese = BaseFont.createFont("STSong-Light", "UniGB-UCS2-H",BaseFont.NOT_EMBEDDED);   
				
				Font FontChinese = new Font(bfChinese, 26, Font.NORMAL);
				Paragraph projectMessage = new Paragraph("["+tenderProjectEntity.getTenderNo() + "]"+tenderProjectEntity.getTenderName(),FontChinese);
				projectMessage.setAlignment(Element.ALIGN_CENTER);
				FontChinese = new Font(bfChinese, 20, Font.NORMAL);
				Paragraph supplierMessage = new Paragraph(supplier.getSupplierName(),FontChinese);
				supplierMessage.setAlignment(Element.ALIGN_CENTER);
				Table table = new Table(2, 5);
				table.setBorderWidth(0f);
				Paragraph p = null;
				Cell cell = null;
				FontChinese = new Font(bfChinese, 14, Font.NORMAL);
				p =  new Paragraph("签到人姓名：",FontChinese);
				cell = new Cell(p);
				cell.setBorderWidth(0f);
				cell.setHorizontalAlignment(Element.ALIGN_RIGHT);
				table.addCell(cell);
				
				p =  new Paragraph(supplier.getSignPerson(),FontChinese);
				cell= new Cell(p);
				cell.setBorderWidth(0f);
				table.addCell(cell);
				
				p =  new Paragraph(Messages.IDCard + "：",FontChinese);
				cell= new Cell(p);
				cell.setBorderWidth(0f);
				cell.setHorizontalAlignment(Element.ALIGN_RIGHT);
				table.addCell(cell);
				
				p =  new Paragraph(supplier.getSignPersonCard(),FontChinese);
				cell= new Cell(p);
				cell.setBorderWidth(0f);
				table.addCell(cell);
				
				p =  new Paragraph("手机号码：",FontChinese);
				cell= new Cell(p);
				cell.setBorderWidth(0f);
				cell.setHorizontalAlignment(Element.ALIGN_RIGHT);
				table.addCell(cell);
				
				p =  new Paragraph(supplier.getSignPersonTel(),FontChinese);
				cell= new Cell(p);
				cell.setBorderWidth(0f);
				table.addCell(cell);
				
				document.add(projectMessage);
				document.add(supplierMessage);
				document.add(table);
				document.close();
			} catch (DocumentException e) {
			} catch (IOException ee) {
			} finally {
				try {
					if (out != null){
						out.close();
					}
				} catch (IOException e1) {
				}
			}
		}
	}
	
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
}
