package com.gx.obe.opening.composite;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.nebula.widgets.grid.Grid;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import com.gx.obe.ColorConstants;
import com.gx.obe.business.enumeration.SupplierEnum;
import com.gx.obe.components.core.I18NEnum;
import com.gx.obe.components.core.enumeration.BidPriceEnum;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.ColumnAlignmentUtils;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.SupplierOpeningResult;
import com.gx.obe.web.entity.price.BidPrice;
import com.gx.obe.web.entity.price.BidPriceResult;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.entity.tender.TenderProjectRuleEntity;
import com.gx.obe.web.service.BidPriceResultService;
import com.gx.obe.web.service.BidPriceService;
import com.swtdesigner.SWTResourceManager;

public class TenderPackBidOpeningDetailComposite extends Composite {

	
	// 开标完成后回调
	private TenderProjectEntity curTenderPack;
	private TenderProjectRuleEntity tenderProjectRuleEntity;
	private List<Supplier> supplierList;
	private List<BidPrice> bidPriceList;
	private Grid bidPriceTableTotalGrid;
	private Map<String, Integer> bidPriceTableTotalColumnMap;
	private BidPriceService bidPriceService;
	private BidPriceResultService bidPriceResultService ;
	private String SUPPLIER_NAME;
	
	
	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */
	public TenderPackBidOpeningDetailComposite(Composite parent, int style) {
		super(parent, style);
		setBackground(ColorConstants.BORDER_LINE_COLOR);

		GridLayout gl_composite_10 = new GridLayout(1, false);
		gl_composite_10.marginWidth = 1;
		gl_composite_10.marginHeight = 1;
		gl_composite_10.verticalSpacing = 0;
		setLayout(gl_composite_10);
		
		Composite composite = new Composite(this, SWT.NONE);
		composite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		GridLayout gl_composite = new GridLayout(1, false);
		gl_composite.marginHeight = 0;
		gl_composite.marginWidth = 0;
		composite.setLayout(gl_composite);
		
		bidPriceTableTotalGrid = new Grid(composite, SWT.H_SCROLL | SWT.V_SCROLL);
		bidPriceTableTotalGrid.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		init();
	}
	
	/**
	 * @Description: 初始化
	 * @author guoyr 
	 */
	private void init(){
		bidPriceService = new BidPriceService();
		bidPriceResultService = new BidPriceResultService();
		bidPriceTableTotalGrid.setHeaderVisible(true);
		bidPriceTableTotalGrid.setItemHeight(50);
		bidPriceTableTotalGrid.setAutoHeight(true);
		bidPriceTableTotalGrid.setShowEvenBackground(false);
		bidPriceTableTotalGrid.setAutoWidth(true);
	}

	/**
	 * @Description: 刷新
	 * @author guoyr
	 * @param curTenderPack
	 * @param tenderProjectRuleEntity
	 * @param supplierList 
	 */
	public void refresh(TenderProjectEntity curTenderPack, TenderProjectRuleEntity tenderProjectRuleEntity, List<Supplier> supplierList){
		this.curTenderPack = curTenderPack;
		this.tenderProjectRuleEntity = tenderProjectRuleEntity;
		this.supplierList = supplierList;
		SUPPLIER_NAME = com.gx.obe.components.core.I18N.getString(I18NEnum.SUPPLIERNAME, curTenderPack.getTenderType());
		if(null == tenderProjectRuleEntity){
			tenderProjectRuleEntity = new TenderProjectRuleEntity();
		}
		
		refreshBidPriceTable();
		refreshPriceTableGrid();
		bidPriceTableTotalGrid.getParent().layout();
		
	}
	/**
	 * @Description: 列出开标一览表总表
	 * @author guoyr 
	 */
	private void refreshBidPriceTable(){
		if(null == bidPriceTableTotalColumnMap){
			bidPriceTableTotalColumnMap = new HashMap<String, Integer>();
		}else {
			bidPriceTableTotalColumnMap.clear();
		}
		if(null != bidPriceList){
			bidPriceList.clear();
		}
		bidPriceList = bidPriceService.getTenderBidPriceTableList(curTenderPack.getId());
		if(null == bidPriceList || bidPriceList.size() == 0){
			bidPriceList = new ArrayList<BidPrice>();
		}
		for(GridColumn column : bidPriceTableTotalGrid.getColumns()){
			column.dispose();
		}
		GridColumn supplierNameColumn = new GridColumn(bidPriceTableTotalGrid, SWT.NONE);
		supplierNameColumn.setHeaderFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.BOLD));
		supplierNameColumn.setText(MessageFormat.format(Messages.NameFormat, SUPPLIER_NAME));
		supplierNameColumn.setWidth(200);
		supplierNameColumn.setWordWrap(true);
		
		for(BidPrice bidPrice : bidPriceList){
			if(StringUtils.isEmpty(bidPrice.getBidPriceName())){
				continue;
			}
			GridColumn gridColumn = new GridColumn(bidPriceTableTotalGrid, SWT.CENTER);;
			if(BidPriceEnum.IS_PRICE_ITEM.equals(bidPrice.getBidPriceType())){
				gridColumn.setAlignment(SWT.RIGHT);
			}else {
				gridColumn.setAlignment(ColumnAlignmentUtils.getAlignment(bidPrice.getBidPriceName()));
				
			}
			gridColumn.setWordWrap(true);
			gridColumn.setHeaderWordWrap(true);
			gridColumn.setText(bidPrice.getBidPriceName());
			gridColumn.setMinimumWidth(100);
			gridColumn.setWidth(100);
			gridColumn.setMaxmumWidth(250);
			bidPriceTableTotalColumnMap.put(bidPrice.getId(), bidPriceTableTotalGrid.indexOf(gridColumn));
		}
	}
	

	/**
	 * @Description: 刷新开标一览表总表
	 * @author guoyr 
	 */
	private void refreshPriceTableGrid(){
		int resultFontSize = 12;//默认字体大小10
		bidPriceTableTotalGrid.removeAll();
		boolean showBidPrice = true;
		boolean onlyShowPrice = false;
		int index = 1;
		for(int i = 0; i < supplierList.size(); i++){
			Supplier supplier = supplierList.get(i);
			if(null == supplier.getTenderTime()){
				continue;
			}
			SupplierOpeningResult supplierOpeningResult = supplier.getSupplierOpeningResult();
			if(null == supplierOpeningResult || null == supplierOpeningResult.getId()){
				continue;
			}
			// 判断投标人是否废标
			if(StringUtils.equals(SupplierEnum.REVOKE_YES, supplierOpeningResult.getRevokeStatus())){
				continue;
			}else {
				// 判断投标人是否已解密成功
				if(null == supplierOpeningResult.getPriceBidStatus() || !supplierOpeningResult.getPriceBidStatus().equals(SupplierEnum.OPENBID_SUCCESS)){
					continue;
				}
			}
			GridItem gridItem = new GridItem(bidPriceTableTotalGrid, SWT.NONE);
			gridItem.setFont(SWTResourceManager.getFont("微软雅黑", resultFontSize, SWT.NONE));
			gridItem.setText(0, index +"、"+supplier.getSupplierName());
			refreshSupplierOpenBidResult(supplier, gridItem, showBidPrice, onlyShowPrice);
			index++;
		}
	}
	private void refreshSupplierOpenBidResult(Supplier supplier, GridItem gridItem, boolean showBidPrice, boolean onlyShowPrice) {
		//获取报价结果
		List<BidPriceResult> bidPriceResultList = bidPriceResultService.getBidPriceResultTableList(curTenderPack.getId(), supplier.getId());
		if(null == bidPriceResultList){
			bidPriceResultList = new ArrayList<BidPriceResult>();
		}
		// 如果投标报价多于一个，则显示总投标报价
		for (BidPriceResult bidPriceResult : bidPriceResultList) {
			if(StringUtils.isEmpty(bidPriceResult.getBidPriceName())){
				continue;
			}
			
			if(BidPriceEnum.IS_PRICE_ITEM.equals(bidPriceResult.getBidPriceType())){
				if(!showBidPrice)continue;
			}else {
				if(onlyShowPrice)continue;
			}
			int columnCount = 1;
			if(null != bidPriceTableTotalColumnMap.get(bidPriceResult.getBidPriceId())){
				columnCount = bidPriceTableTotalColumnMap.get(bidPriceResult.getBidPriceId());
				gridItem.setText(columnCount, bidPriceResult.getContentValue(bidPriceResult.getPriceUnit(tenderProjectRuleEntity.getPriceUnitUseDefault())));
			}
		}
	}
	
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
	

}
