package com.gx.obe.opening.dialog;

import java.text.MessageFormat;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.utils.SwtUtils;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.core.widgets.ImageButton;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.components.core.Constants;
import com.gx.obe.message.Messages;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 
 * @author guoyr
 */
public class SelectDecryptTypeDialog extends EDialog {
	private ImageButton confirmBtn;
	private String[] openingTypeArray;
	private Image loginBgImage;
	private Composite managerTypeComposite;
	private String selectOpeningType;

	/**
	 * Create the dialog.
	 * @param parentShell
	 * @param callback 
	 */
	public SelectDecryptTypeDialog(Shell parentShell, String[] openingTypeArray) {
		super(parentShell);
		this.openingTypeArray = openingTypeArray;
		this.setOperator(SWT.NONE);
		setTitleImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/white/law.png"));
		setTitle(MessageFormat.format(Messages.Tips_pleaseSelect, Messages.Button_decryptType));
	}

	/**
	 * Create contents of the dialog.
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		GridLayout gl_container = new GridLayout(1, false);
		gl_container.marginHeight = 10;
		gl_container.marginWidth = 10;
		container.setLayout(gl_container);
		
		
		managerTypeComposite = new Composite(container, SWT.NONE);
		managerTypeComposite.setBackgroundMode(SWT.INHERIT_FORCE);
		GridLayout gl_managerTypeComposite = new GridLayout(1, false);
		gl_managerTypeComposite.marginWidth = 50;
		gl_managerTypeComposite.marginHeight = 20;
		managerTypeComposite.setLayout(gl_managerTypeComposite);
		GridData gd_managerTypeComposite = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_managerTypeComposite.minimumHeight = 100;
		gd_managerTypeComposite.minimumWidth = 200;
		managerTypeComposite.setLayoutData(gd_managerTypeComposite);
		
		Composite composite_1 = new Composite(container, SWT.NONE);
		composite_1.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, false, 1, 1));
		GridLayout gl_composite_1 = new GridLayout(1, false);
		gl_composite_1.marginHeight = 0;
		gl_composite_1.marginWidth = 0;
		composite_1.setLayout(gl_composite_1);
		
		confirmBtn = new ImageButton(composite_1, SWT.NONE);
		confirmBtn.setForeground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		confirmBtn.setFont(SWTResourceManager.getFont("微软雅黑", 11, SWT.BOLD));
		GridData gd_confirmBtn = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_confirmBtn.heightHint = 30;
		confirmBtn.setMinWidth(90);
		confirmBtn.setLeftMargin(10);
		confirmBtn.setRightMargin(10);
		confirmBtn.setLayoutData(gd_confirmBtn);
		confirmBtn.setText(Messages.Button_confirm);
		confirmBtn.setBackground(ColorConstants.SKIN_BG);
		confirmBtn.setActiveBgColor(ColorConstants.SKIN_ACTIVE_BG);
//		confirmBtn.setBackgroundImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/login/button.png"));
//		confirmBtn.setActiveBgImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/login/buttonhover.png"));
		confirmBtn.setActiveForeground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		confirmBtn.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				confirmAction();
			}
		});
//		confirmBtn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/user.png"));

		init();
		return container;
	}
	
	/**
	 * @Description: 
	 * @author guoyr
	 */
	private void init(){
		if(null != openingTypeArray){
			for(String openingType : openingTypeArray){
				addDecBidTypeBtn(managerTypeComposite, openingType);
			}
		}
		getShell().pack();
		SwtUtils.setCenter(getShell());
	}
	

	
	

	/**
	 * @Description: 
	 * @author guoyr
	 */
	private void confirmAction(){
		close();
	}

	@Override
	public boolean close() {
		if(null != loginBgImage && !loginBgImage.isDisposed()){
			loginBgImage.dispose();
			loginBgImage = null;
		}
		return super.close();
	}

	private ImageButton addDecBidTypeBtn(final Composite parent, final String openingType){
		final ImageButton label = new ImageButton(parent, SWT.NONE);
		if(Constants.DECRYPT_LOCAL.equals(openingType)){
			label.setText(Messages.Button_decryptLocal);
//		}else if(Constants.DECRYPT_LOCAL_SELF .equals(openingType)){
//			label.setText("现场自助机解密");
			
		}else if(Constants.DECRYPT_REMOTE .equals(openingType)){
			label.setText(Messages.Button_decryptRemote);
		}else if(Constants.DECRYPT_FOCUS.equals(openingType)){
			label.setText(Messages.Button_decryptCentralize);
		}
			
		label.setLeftMargin(20);
		label.setRightMargin(20);
		label.setSubImageRightMargin(5);
		label.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, true, 1, 1));
		label.setData("openingType", openingType);
		label.setSelectionForeground(SWTResourceManager.getColor(77, 163, 19));
		label.setSelection(false);
		label.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/raido_unchecked.png"));
		label.setForeground(ColorConstants.FONT_BLACK_COLOR);
		label.setFont(SWTResourceManager.getFont("微软雅黑", 12, SWT.NORMAL));
		label.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				selectOpeningType = null;
				for(Control control : parent.getChildren()){
					if(control.equals(label)){
						continue;
					}
					ImageButton button = (ImageButton)control;
					button.setSelection(false);
					button.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/raido_unchecked.png"));
				}
				if(label.getSelection()){
//					label.setSelection(false);
//					label.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/raido_unchecked.png"));
				}else {
					label.setSelection(true);
					selectOpeningType = openingType;
					label.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/radio_checked.png"));
				}
			}
		});
		return label;
	}
	

	public String getSelectOpeningType() {
		return selectOpeningType;
	}

	public void setSelectOpeningType(String selectOpeningType) {
		this.selectOpeningType = selectOpeningType;
	}

	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(450, 300);
	}
}
