package com.gx.obe.opening.dialog;

import org.eclipse.nebula.widgets.grid.Grid;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.message.Messages;
import com.gx.obe.opening.listener.SelectDeviceListener;
import com.gx.obe.util.utils.LogUtils;
import com.swtdesigner.ResourceManager;

/**
 * @Description: 选择视频设备界面
 * @author guoyr
 */
public class SelectEquipmentDialog extends EDialog {
	private EButton saveBtn;
	private long deviceNo;
	private Grid grid;
	private SelectDeviceListener listener;

	{
		LogUtils.logClass(SelectEquipmentDialog.class);
	}

	/** 
	 * @author guoyr
	 * @param parentShell
	 * @param tenderProjectEntity
	 * @param supplier 
	 */
	public SelectEquipmentDialog(Shell parentShell, long deviceNo, SelectDeviceListener listener) {
		super(parentShell);
		setTitle("选择视频设备");
		this.deviceNo = deviceNo;
		this.listener = listener;
	}

	/**
	 * @Description: 
	 * @author guoyr
	 * @param parent
	 * @return 
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		
		Composite top = (Composite) super.createDialogArea(parent);
		GridLayout gl_top = new GridLayout(1, false);
		gl_top.verticalSpacing = 0;
		gl_top.horizontalSpacing = 0;
		gl_top.marginHeight = 0;
		gl_top.marginWidth = 0;
		top.setLayout(gl_top);
		
		Composite container = new Composite(top, SWT.NONE);
		container.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		GridLayout gl_container = new GridLayout(1, false);
		container.setLayout(gl_container);
		
		grid = new Grid(container, SWT.BORDER);
		
		grid.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		grid.setHeaderVisible(true);
		GridColumn column = new GridColumn(grid, SWT.CENTER);
		column.setText(Messages.Num);
		column.setWidth(40);
		column.setMaxmumWidth(40);
		
		GridColumn equipmentName_column = new GridColumn(grid, SWT.CENTER);
		equipmentName_column.setText("设备名称");
		column.setWidth(200);
		column.setMinimumWidth(200);
		
		ESepator label_1 = new ESepator(top, SWT.SEPARATOR | SWT.HORIZONTAL);
		label_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		label_1.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		
		Composite composite_1 = new Composite(top, SWT.NONE);
		composite_1.setBackgroundMode(SWT.INHERIT_FORCE);
		composite_1.setBackgroundImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/fill_bg.png"));
		composite_1.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, false, 1, 1));
		GridLayout gl_composite_1 = new GridLayout(1, false);
		composite_1.setLayout(gl_composite_1);
		
		saveBtn = new EButton(composite_1, SWT.NONE);
		saveBtn.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				saveAction();
			}
		});
		GridData gd_importFileBtn = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_importFileBtn.widthHint = 90;
		saveBtn.setLayoutData(gd_importFileBtn);
		saveBtn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/success.png"));
		saveBtn.setText(Messages.Button_ok);

		init();
		return container;
	}
	
	/**
	 * @Description: 界面数据初始化，列举视频设备
	 * @author guoyr 
	 */
	private void init(){
		grid.addMouseListener(new MouseAdapter() {
			public void mouseDoubleClick(MouseEvent e) {
				listener.selectVideoNo(grid.getSelectionIndex());
				getShell().close();
			}
			public void mouseUp(MouseEvent e) {
				saveBtn.setEnabled(grid.getSelectionCount() > 0);
			}
		});
		if(deviceNo > 0){
			for(int i = 0; i < deviceNo; i++){
				GridItem item = new GridItem(grid, SWT.NONE);
				item.setText(0, (i + 1) + "");
				item.setText(1, "设备" + (i + 1));
				item.setData("deviceNo", i);
			}
			
		}
	}

	protected Point getInitialSize() {
		return new Point(413, 263);
	}
	
	/**
	 * @Description: 保存
	 * @author guoyr 
	 */
	private void saveAction(){
		listener.selectVideoNo(grid.getSelectionIndex());
		getShell().close();
	}
}
