package com.gx.obe.opening.dialog;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.file.entity.BidFile;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.common.widget.button.BidFileButton;
import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.util.utils.LogUtils;
import com.gx.obe.util.utils.ScreenUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.tools.ole.DocumentFactory;
import com.gx.tools.ole.DocumentLoader;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

/** 
 * @Description: 查看开标一览表
 * @author guoyr 
 */
public class ViewBidFilesDialog extends EDialog {

	private Composite pdfLoadComposite;
	private ScrolledComposite scrolledComposite;
	private Composite fileList_composite;
	private Composite bidFile_composite;
	private DocumentLoader documentLoader;
	private String tempStuffix = "";
	private String signaturePdf = PropertiesUtils.getProperty(ConfigConstants.PDF_ACTIVE_TYPE, "");
	private BidFileButton curBidFileButton;
	private List<BidFile> bidPriceFileList;
	private String PDF_SIGN = "pdf";
	private String EXCEL_SIGN = "xls";
	private String WORD_SIGN = "doc";
	private TenderProjectEntity tenderProjectEntity;
	private Supplier supplier;
	
	{
		LogUtils.logClass(ViewBidFilesDialog.class);
	}
	
	/** 
	 * @author guoyr
	 * @param parentShell
	 * @param tenderProjectEntity
	 * @param templateType 
	 */
	public ViewBidFilesDialog(Shell parentShell, TenderProjectEntity tenderProjectEntity, Supplier supplier) {
		super(parentShell);
		setTitle("标书列表");
		this.tenderProjectEntity = tenderProjectEntity;
		this.supplier = supplier;
	}

	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		GridLayout gl_container = new GridLayout(3, false);
		gl_container.verticalSpacing = 0;
		gl_container.horizontalSpacing = 0;
		gl_container.marginWidth = 0;
		gl_container.marginHeight = 0;
		container.setLayout(gl_container);
		
		scrolledComposite = new ScrolledComposite(container, SWT.BORDER | SWT.V_SCROLL);
		GridData gd_scrolledComposite = new GridData(SWT.FILL, SWT.FILL, false, true, 1, 3);
		gd_scrolledComposite.widthHint = 150;
		scrolledComposite.setLayoutData(gd_scrolledComposite);
		scrolledComposite.setExpandHorizontal(true);
		scrolledComposite.setExpandVertical(true);
		
		fileList_composite = new Composite(scrolledComposite, SWT.NONE);
		fileList_composite.setLayout(new GridLayout(1, false));
		
		bidFile_composite = new Composite(fileList_composite, SWT.NONE);
		GridLayout gl_sinatureFile_composite = new GridLayout(1, false);
		gl_sinatureFile_composite.verticalSpacing = 2;
		gl_sinatureFile_composite.horizontalSpacing = 0;
		gl_sinatureFile_composite.marginWidth = 0;
		gl_sinatureFile_composite.marginHeight = 0;
		bidFile_composite.setLayout(gl_sinatureFile_composite);
		GridData gd_sinatureFile_composite = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
		gd_sinatureFile_composite.widthHint = 107;
		bidFile_composite.setLayoutData(gd_sinatureFile_composite);
		
		
		scrolledComposite.setContent(fileList_composite);
		scrolledComposite.setMinSize(fileList_composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		
		ESepator label = new ESepator(container, SWT.VERTICAL);
		label.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, true, 1, 3));
		label.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		
		pdfLoadComposite = new Composite(container, SWT.NONE);
		pdfLoadComposite.setForeground(SWTResourceManager.getColor(60, 138, 226));
		pdfLoadComposite.setFont(SWTResourceManager.getFont("微软雅黑", 16, SWT.BOLD));
		pdfLoadComposite.setBackground(SWTResourceManager.getColor(240, 240, 240));
		pdfLoadComposite.setLayout(new FillLayout(SWT.HORIZONTAL));
		pdfLoadComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		ESepator label_1 = new ESepator(container, SWT.NONE);
		label_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		label_1.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		init();
		return container;
	}
	
	/**
	 * @Description: 初始化
	 * @author guoyr 
	 */
	private void init(){
		bidPriceFileList = BidFile.getSupplierBidFileList(tenderProjectEntity.getId(), supplier.getId(), null);
		refreshUserSinatureFiles();
	}
	/**
	 * @Description: 刷新供应商解密后投标文件
	 * @author guoyr 
	 */
	private void refreshUserSinatureFiles(){
		for(Control control : bidFile_composite.getChildren()){
			control.dispose();
		}
		
		if(null != bidPriceFileList && bidPriceFileList.size() > 0){
			for(int i =  bidPriceFileList.size() ; i > 0; i--){
				final BidFile bidFile = bidPriceFileList.get(i - 1);
				final BidFileButton bidFileButton = new BidFileButton(bidFile_composite, SWT.NONE);

				bidFileButton.setText(bidFile.getSimpleName());
				if(null != bidFile.getSuffix() && bidFile.getSuffix().toLowerCase().contains(PDF_SIGN)){
					bidFileButton.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/pdf_64.png"));
				}else if(null != bidFile.getSuffix() && bidFile.getSuffix().toLowerCase().contains(EXCEL_SIGN)){
					bidFileButton.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/excel_64.png"));
				}else if(null != bidFile.getSuffix() && bidFile.getSuffix().toLowerCase().contains(WORD_SIGN)){
					bidFileButton.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/word_64.png"));
				}else{
					bidFileButton.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/unknown_64.png"));
				}
				if(null != bidFile.getFileType() && bidFile.getFileType().equals(BidFile.TYPE_BIDPRICE)){//当前为报价文件，在控件下多添加一个标志图片
					bidFileButton.setSubImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/money-yen.png"));
				}
				GridData gd_signatureFileButton = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
				gd_signatureFileButton.widthHint = 80;
				bidFileButton.setLayoutData(gd_signatureFileButton);
				bidFileButton.addMouseListener(new MouseAdapter() {
					public void mouseUp(MouseEvent e) {
						reflashButtonStatus(bidFileButton);
						loadSignatureFile(bidFile);
					}
				});
				//默认选择加载第二个
				if(i ==  bidPriceFileList.size()){
					reflashButtonStatus(bidFileButton);
				}
			}
			loadSignatureFile(bidPriceFileList.get(bidPriceFileList.size() - 1));
			
		}
		
		bidFile_composite.layout();
		fileList_composite.layout();
		scrolledComposite.setMinSize(fileList_composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}

	
	/**
	 * @Description: 加载签章
	 * @author guoyr 
	 */
	private void loadSignatureFile(BidFile bidFile){
		if(null!= bidFile.getSuffix() && bidFile.getSuffix().contains(EXCEL_SIGN) || bidFile.getSuffix().contains(PDF_SIGN) || bidFile.getSuffix().contains(WORD_SIGN)){//只加载office文件
			//当发现类型与之前打开不一致，先把整个控件面板全部注销，重新打开
			if(!tempStuffix.equals(bidFile.getSuffix())){
				for(Control control : pdfLoadComposite.getChildren()){
					control.dispose();
				}
				if(null != documentLoader){
					documentLoader.close();
					documentLoader = null;
				}
			}
			if(null == documentLoader){
				documentLoader = DocumentFactory.getDocumentLoader(bidFile.getAbsolutePath(), signaturePdf);
				documentLoader.openDocument(pdfLoadComposite, false, false, false, false, false);
			}else{
				documentLoader.openFile(bidFile.getAbsoluteFile());
			}
			pdfLoadComposite.layout();
			tempStuffix = bidFile.getSuffix();
		}
		
		
	}
	@Override
	public boolean close() {
		return super.close();
	}
	/**
	 * @Description: 刷新左侧选择状态
	 * @author guoyr
	 * @param bidFileButton 
	 */
	private void reflashButtonStatus(BidFileButton bidFileButton){
		for(Control control : bidFile_composite.getChildren()){
			BidFileButton eachButton = (BidFileButton)control;
			eachButton.setSelection(false);
		}
		curBidFileButton = bidFileButton;
		curBidFileButton.setSelection(true);
		
	}
	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		ScreenUtils  util = new ScreenUtils();
		int screenHight  = util.getScreenHight() -util.getTaskBarHight();
		return new Point(util.getScreenWight(),screenHight);
	}
}
