package com.gx.obe.opening.thread;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.common.widget.thread.MyRunnable;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.opening.listener.BatchRemoteSignListener;
import com.gx.obe.web.entity.TransactionServer;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.entity.tender.TenderProjectRuleEntity;
import com.gx.obe.web.service.TenderProjectService;
import com.gx.obe.web.vo.TenderPackOpeningInfo;

public class BatchDownRemoteSignInfoThread extends MyRunnable{

	
	private Thread thread;
	private TenderProjectEntity tenderProjectEntity;
	private String bidopeningStage;
	
	private BatchRemoteSignListener remoteProjectSignListener;
	private TenderProjectService tenderProjectService ;
	private RemoteProjectSignInfoThread remoteProjectSignInfoThread;

	
	public BatchDownRemoteSignInfoThread(TenderProjectEntity tenderProjectEntity, TenderProjectRuleEntity tenderProjectRuleEntity, String bidopeningStage, BatchRemoteSignListener remoteProjectSignListener){
		this.tenderProjectEntity = tenderProjectEntity;
		this.bidopeningStage = bidopeningStage;
		this.remoteProjectSignListener = remoteProjectSignListener;
		tenderProjectService = new TenderProjectService();
	}
	
	
	public void start(){
		thread = new Thread(this);
		thread.setName(tenderProjectEntity.getTenderName()+"[下载签到信息]" + bidopeningStage);
		thread.start();
	}
	
	public void stop(){
		thread = null;
		if(null != remoteProjectSignInfoThread){
			remoteProjectSignInfoThread.stop();
			remoteProjectSignInfoThread = null;
		}
	}
	public void join(){
		if(null != thread){
			try {
				thread.join();
			} catch (InterruptedException e) {
			}
		}
	}
	public void run() {
		List<TenderPackOpeningInfo> tenderPackOpeningInfoList = null;
		TransactionServer transactionServer = HttpUtils.getTransactionServer(tenderProjectEntity.getTransactionServerId());
		boolean isPriceStage = false;
		boolean isBusAndTecStage = false;
		if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
			isPriceStage = true;
		}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
			isBusAndTecStage = true;
		}
		long startTime = Calendar.getInstance().getTimeInMillis();
		long useTime = -1;
		while(null != thread){
			startTime = Calendar.getInstance().getTimeInMillis();
			if(null != tenderPackOpeningInfoList){
				tenderPackOpeningInfoList.clear();
				tenderPackOpeningInfoList = null;
			}
			tenderPackOpeningInfoList = tenderProjectService.getTenderPackOpeningInfoList(tenderProjectEntity.getId(), bidopeningStage);
			if(null != tenderPackOpeningInfoList && tenderPackOpeningInfoList.size() > 0){
				List<TenderProjectEntity> getSingInfoList = new ArrayList<TenderProjectEntity>();
				for(TenderPackOpeningInfo tenderPackOpeningInfo : tenderPackOpeningInfoList){
					TenderProjectEntity tenderPack = tenderPackOpeningInfo.getTenderProject();
					
					String startBidOpening = tenderPack.getStartBidOpening();
					if(isPriceStage){
						startBidOpening = tenderPack.getStartPriceBidOpening();
					}
					
//					if(tenderPack.getTenderNo().contains("9-2")){
//						getSingInfoList.add(tenderPack);
//					}
					// 只选择已启动的，只要项目没完成，则一直获取签到信息（签到不做为解密的必要条件）
					if(null != startBidOpening && startBidOpening.equals(CommonEnum.YES)){
//						if(isPriceStage){
//							// 如果已开始解密，则不再获取签到信息
//							if(null != tenderPack.getPriceBidStatus() && (tenderPack.getPriceBidStatus().equals(CommonEnum.IN_PROGRESS) ||tenderPack.getPriceBidStatus().equals(CommonEnum.COMPLETE))){
//								continue;
//							}
//						}else {
//							// 如果已开始解密，则不再获取签到信息
//							if(null != tenderPack.getBusinessBidStatus() && (tenderPack.getBusinessBidStatus().equals(CommonEnum.IN_PROGRESS) ||tenderPack.getBusinessBidStatus().equals(CommonEnum.COMPLETE))){
//								continue;
//							}
//						}
						String bidStatus = tenderPack.getBusinessBidStatus();
						if(isPriceStage){
							bidStatus = tenderPack.getPriceBidStatus();
						}
						// 如果已经完成，则不再获取签到信息
						if(null != bidStatus && bidStatus.equals(CommonEnum.COMPLETE)){
							remoteProjectSignListener.refreshSignInfo(tenderPackOpeningInfo);
							// 因为签到线程和获取解密线程是异步，且每10秒才获取一次，避免先获取到开标结果，且完成开标后未获取签到信息
							if(tenderPackOpeningInfo.getDecCount() > tenderPackOpeningInfo.getSignCount()) {
								getSingInfoList.add(tenderPack);
							}
							continue;
						}
						if(!tenderPackOpeningInfo.isSignComplate()){
							getSingInfoList.add(tenderPack);
						}else {
							remoteProjectSignListener.refreshSignInfo(tenderPackOpeningInfo);
						}
					}
				}
				
				if(null != thread && getSingInfoList.size() > 0){
					if(null != remoteProjectSignInfoThread){
						remoteProjectSignInfoThread.stop();
						remoteProjectSignInfoThread = null;
					}
					// 每隔10秒刷新
					if(useTime > 10000){
						// 此次刷新时间超过10秒
					}else if(useTime > 0) {
						try {
							Thread.sleep(10000 - useTime);
						} catch (InterruptedException e) {
						}
					}
					// 下载签到信息
					remoteProjectSignInfoThread = new RemoteProjectSignInfoThread(tenderProjectEntity, bidopeningStage, transactionServer, getSingInfoList, remoteProjectSignListener);
					remoteProjectSignInfoThread.start();
					
				}else {
					thread = null;
				}
				
			}
			
			// 下载完签信息和解密信息后才能刷新
			if(null != remoteProjectSignInfoThread){
				remoteProjectSignInfoThread.join();
			}
			useTime = Calendar.getInstance().getTimeInMillis() - startTime;
//			System.out.println("下载签到用时" + useTime);
		}
	}
	
}
