package com.gx.obe.opening.thread;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.apache.log4j.Logger;

import com.gx.obe.business.enumeration.SupplierEnum;
import com.gx.obe.listener.BidPriceResolveListener;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.SupplierOpeningResult;
import com.gx.obe.web.entity.TransactionServer;
import com.gx.obe.web.entity.price.BidPrice;
import com.gx.obe.web.entity.price.BidPriceResult;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.BidPriceService;
import com.gx.obe.web.service.SupplierOpeningResultService;
import com.gx.obe.web.service.SupplierService;
import com.gx.tools.utils.BidPriceUtils;

public class BatchDownloadBidPriceResultThread implements Runnable{
	
	private Logger LOG = Logger.getLogger(RemoteProjectDecryptInfoThread.class);
	private TenderProjectEntity tenderProjectEntity;
	private TransactionServer transactionServer;
	private List<TenderProjectEntity> tenderPackList;
	private String bidopeningStage;
	private Thread thread;
	private ThreadPoolExecutor downloadBidPriceThreadPool;
	private SupplierService supplierService = new SupplierService();
	private BidPriceService bidPriceService ;
	private SupplierOpeningResultService supplierOpeningResultService;
	

	public BatchDownloadBidPriceResultThread(TenderProjectEntity tenderProjectEntity, String bidopeningStage, TransactionServer transactionServer, List<TenderProjectEntity> tenderPackList){
		this.tenderProjectEntity = tenderProjectEntity;
		this.bidopeningStage = bidopeningStage;
		this.transactionServer = transactionServer;
		this.tenderPackList = tenderPackList;
		bidPriceService = new BidPriceService();
		supplierOpeningResultService  = new SupplierOpeningResultService();
	}
	public void start(){
		thread = new Thread(this);
		thread.start();
	}
	public void stop(){
		thread = null;
		if(null != downloadBidPriceThreadPool){
			downloadBidPriceThreadPool.shutdownNow();
		}
	}
	public void join(){
		if(null != thread){
			try {
				thread.join();
			} catch (InterruptedException e) {
			}
		}
	}
	
	public void run() {
		
		int poolSize = tenderPackList.size() > 10 ? 10 :tenderPackList.size();
		downloadBidPriceThreadPool = new ThreadPoolExecutor(poolSize, poolSize, 3, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>(),new ThreadPoolExecutor.CallerRunsPolicy());
		List<Future<?>> allTenderPackFutureList = new ArrayList<Future<?>>();
		for(final TenderProjectEntity tenderPack : tenderPackList){
			
			loadBidPrice(tenderPack);
			final List<Supplier> supplierList = supplierService.getSupplierOpenBidList(tenderPack.getId());
			final BidPriceResolveListener bidPriceResolveListener = new BidPriceResolveListener(tenderPack, supplierList.size() > 10 ? 10 : supplierList.size());
			allTenderPackFutureList.add(downloadBidPriceThreadPool.submit(new Callable<Object>() {
				public Object call() throws Exception {
					if(null != supplierList && supplierList.size() > 0){
						for(final Supplier supplier : supplierList){
							final SupplierOpeningResult supplierOpeningResult = supplier.getSupplierOpeningResult();
							// 如果废标了，则不需要获取
							if(null != supplierOpeningResult.getRevokeStatus() && supplierOpeningResult.getRevokeStatus().equals(SupplierEnum.REVOKE_YES)){
								continue;
							}
							if(null == thread){
								break;
							}
							// 如果价格标开标，且没有报价，则下载报价
							if(null != supplierOpeningResult.getPriceBidStatus() && supplierOpeningResult.getPriceBidStatus().equals(SupplierEnum.OPENBID_SUCCESS) && null == supplierOpeningResult.getBidPrice()){
								Future<List<BidPriceResult>> future = bidPriceResolveListener.downloadRemoteBidOpeningResult(supplier);
								try {
									List<BidPriceResult> bidPriceResultList = future.get();
									
									if(null != bidPriceResultList && null != thread){
										// 下载投标报价后保存投标报价
										bidPriceResolveListener.saveSupplierBidPriceResult(supplierOpeningResult, null, null, supplierOpeningResult.getPriceBidStatus(), SupplierEnum.OPENBID_SUCCESS, null, null, bidPriceResultList);
									}
								} catch (InterruptedException e) {
									LOG.error("", e);
								} catch (ExecutionException e) {
									LOG.error("", e);
								}
							}
						}
					}
					return null;
				}
				
			}));
		}
		downloadBidPriceThreadPool.shutdown();
		for(Future<?> future : allTenderPackFutureList){
			if(null == thread){
				break;
			}
			try {
				future.get();
			} catch (InterruptedException e) {
			} catch (ExecutionException e) {
			}
		}
		
	}
	
	/**
	 * @Description: 先加载开标一览表结构
	 * @author guoyr 
	 */
	private void loadBidPrice(TenderProjectEntity tenderPack){
		// 开标解密时先判断是否已经存在报价结构
		if(bidPriceService.getBidPriceTableCount(tenderPack.getId()) == 0){
			// 如果不存在报价结构，则从招标文件中读取结构
			List<BidPrice> bidPriceTable = BidPriceUtils.getBidPriceTable(tenderPack.getId(), tenderPack.getTenderNo());
			if(null != bidPriceTable && bidPriceTable.size() > 0){
				bidPriceService.batchSaveResolverBidPriceTable(bidPriceTable);
			}
		}
	}
}
