package com.gx.obe.opening.thread;

import java.util.Date;
import java.util.List;
import java.util.Properties;

import com.gx.obe.VideoConfigConstants;
import com.gx.obe.WebserviceConstants;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.common.widget.listener.ProgressMessageListener;
import com.gx.obe.common.widget.thread.MyRunnable;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.DateUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.web.entity.BidOpeningRoom;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.BidOpeningRoomService;
import com.gx.obe.web.service.SupplierService;
import com.gx.obe.web.service.TenderProjectService;
import com.gx.utils.ActionLogUtils;
import com.gx.utils.Params;

/** 
 * @Description: 批量远程解密
 * @author guoyr 
 */
public class BatchFocusDecryptBidFilThread extends MyRunnable{
	
	private TenderProjectEntity tenderProjectEntity;
	private String bidopeningStage;
	private Date bidOpeningTime;
	private Date signEndTime;
	private Date decEndTime;
	private ProgressMessageListener callBack;
	private List<TenderProjectEntity> tenderPackList;
	private Thread thread;
	private final String TIMESTAMP_STR = "yyyy-MM-dd HH:mm:ss";
	private SupplierService supplierService = new SupplierService();
	private boolean synchronizedInfoOnly = false;

	public BatchFocusDecryptBidFilThread(TenderProjectEntity tenderProjectEntity, List<TenderProjectEntity> tenderPackList, String bidopeningStage, Date bidOpeningTime, Date signEndTime, Date decEndTime, boolean synchronizedInfoOnly, ProgressMessageListener callBack){
		this.tenderProjectEntity = tenderProjectEntity;
		this.tenderPackList = tenderPackList;
		this.bidopeningStage = bidopeningStage;
		this.bidOpeningTime = bidOpeningTime;
		this.signEndTime = signEndTime;
		this.decEndTime = decEndTime;
		this.synchronizedInfoOnly = synchronizedInfoOnly;
		this.callBack = callBack;

	}
	public void start(){
		thread = new Thread(this);
		thread.start();
	}
	public void stop(){
		thread = null;
	}
	public void join(){
		if(null != thread){
			try {
				thread.join();
			} catch (InterruptedException e) {
			}
		}
	}
	public void run(){

		boolean startRemoteDecrypt = false;
		String openStageType = "00";
		boolean isPriceStage = false;
		boolean isBusAndTecStage = false;
		if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
			isPriceStage = true;
			openStageType = "02";
		}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
			openStageType = "01";
			isBusAndTecStage = true;
		}
	

		// 只同步解密信息
		if(synchronizedInfoOnly){
			startRemoteDecrypt = true;
			ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"批量同步集中解密信息", new Params("tenderNo", tenderProjectEntity.getTenderNo()).add("openStageType", openStageType));
		}else {
			callBack.message(Messages.Tips_starting, 0);
			HttpUrl httpUrl = HttpUtils.getProjectHttpUrl(tenderProjectEntity, PropertiesUtils.getWebserviceProperty(WebserviceConstants.START_FOCUS_BIDOPENING_METHOD, ""));
			if(null != httpUrl){
				httpUrl.setName("批量启动集中解密");
				httpUrl.setAttribute("openStageType", openStageType);
				if(null != tenderPackList){
					StringBuilder stageIds = new StringBuilder();
					for(TenderProjectEntity tenderPack : tenderPackList){
						if(stageIds.length() > 0){
							stageIds.append(",");
						}
						stageIds.append(tenderPack.getStageId());
					}
					httpUrl.setAttribute("stageIds", stageIds.toString());
				}
				
				httpUrl.setAttribute("openStageType", openStageType);
				Date bidOpeningSignDeadLine = null != signEndTime ? signEndTime : bidOpeningTime;
				httpUrl.setAttribute("bidOpeningSignDeadLine", null != bidOpeningSignDeadLine ? DateUtils.getFormat(bidOpeningSignDeadLine, TIMESTAMP_STR) : null);// 签到截止时间
				httpUrl.setAttribute("bidOpeningTime", DateUtils.getFormat(bidOpeningTime, TIMESTAMP_STR));//开标开标时间
				httpUrl.setAttribute("bidOpeningDeadLine", null != decEndTime ? DateUtils.getFormat(decEndTime, TIMESTAMP_STR) : null);// 解密截止时间
				String bidOpeningRoomAddress = "";
				if(null != tenderProjectEntity.getBidOpeningRoomId()){
					BidOpeningRoomService bidOpeningRoomService = new BidOpeningRoomService();
					BidOpeningRoom bidOpeningRoom = bidOpeningRoomService.getBidOpeningRoomById(tenderProjectEntity.getBidOpeningRoomId());
					if(null != bidOpeningRoom){
						bidOpeningRoomAddress = bidOpeningRoom.getBidOpeningRoomAddress();
					}
				}
				if(null == bidOpeningRoomAddress || bidOpeningRoomAddress.length() == 0 && null != tenderProjectEntity.getOpeningAddress()){
					bidOpeningRoomAddress = tenderProjectEntity.getOpeningAddress();
				}
				Properties videoProperties = PropertiesUtils.getProperties("videoConfig.properties");
				if(CommonEnum.LIVING_COM_OTHER.equals(videoProperties.getProperty(VideoConfigConstants.LIVING_TYPE, CommonEnum.LIVING_COM_OTHER))){
					String videoLiveUrl = ObjectUtils.getObjString(videoProperties.getProperty(VideoConfigConstants.VIDEO_LIVE_URL));
					String videoDeskLiveUrl = ObjectUtils.getObjString(videoProperties.getProperty(VideoConfigConstants.VIDEO_DESK_LIVE_URL));
					httpUrl.setAttribute("bidOpeningLiveCameraUrl", videoLiveUrl);
					httpUrl.setAttribute("bidOpeningLiveDesktopUrl", videoDeskLiveUrl);
				}else{
//					httpUrl.setAttribute("bidOpeningLiveCameraUrl", VideoLiveShell.getCameraVideoLiveUrl(tenderNo));
//					httpUrl.setAttribute("bidOpeningLiveDesktopUrl", VideoLiveShell.getDeskVideoLiveUrl(tenderNo));
				}
			}
			startRemoteDecrypt = HttpUtils.doPost(httpUrl).getStatus();
			ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"批量启动集中解密", httpUrl, startRemoteDecrypt);
			if(!startRemoteDecrypt) {
				startRemoteDecrypt = HttpUtils.doPost(httpUrl).getStatus();
				ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"第二次批量启动集中解密", httpUrl, startRemoteDecrypt);
			}
		}
		

		TenderProjectService tenderProjectService = new TenderProjectService();
		if(startRemoteDecrypt){
			for(TenderProjectEntity tenderPack : tenderPackList){
				// 初始化供应商开标结果
				supplierService.initSupplierBidOpeningResult(tenderPack.getId());
				if(null == tenderPack.getBidOpeningStatus() || !tenderPack.getBidOpeningStatus().equals(CommonEnum.COMPLETE)){
					tenderPack.setBidOpeningStatus(CommonEnum.IN_PROGRESS);
				}
				if(isPriceStage){
					tenderPack.setStartPriceBidOpening(CommonEnum.YES);
					tenderPack.setPriceBidStatus(CommonEnum.IN_PROGRESS);
				}else if(isBusAndTecStage){
					tenderPack.setStartBidOpening(CommonEnum.YES);
					tenderPack.setBusinessBidStatus(CommonEnum.IN_PROGRESS);
					tenderPack.setTechnologyBidStatus(CommonEnum.IN_PROGRESS);
				}else{
					tenderPack.setStartBidOpening(CommonEnum.YES);
					tenderPack.setBusinessBidStatus(CommonEnum.IN_PROGRESS);
					tenderPack.setTechnologyBidStatus(CommonEnum.IN_PROGRESS);
					tenderPack.setPriceBidStatus(CommonEnum.IN_PROGRESS);
				}
			}
			if(isPriceStage){
				tenderProjectService.batchUpdateProperty(tenderPackList, new String[]{"startPriceBidOpening", "priceBidStatus", "bidOpeningStatus"});
			}else if(isBusAndTecStage){
				tenderProjectService.batchUpdateProperty(tenderPackList, new String[]{"startBidOpening", "businessBidStatus", "technologyBidStatus", "bidOpeningStatus"});
			}else{
				tenderProjectService.batchUpdateProperty(tenderPackList, new String[]{"startBidOpening", "priceBidStatus", "businessBidStatus", "technologyBidStatus", "bidOpeningStatus"});
			}
			if(isPriceStage){
				tenderProjectEntity.setStartPriceBidOpening(CommonEnum.YES);
				tenderProjectService.updateAssignProperty(tenderProjectEntity, new String[]{"id","startPriceBidOpening"});
			}else {
				tenderProjectEntity.setStartBidOpening(CommonEnum.YES);
				tenderProjectService.updateAssignProperty(tenderProjectEntity, new String[]{"id","startBidOpening"});
			}
			tenderProjectService.updateBidOpeningStatus(tenderProjectEntity, bidopeningStage, CommonEnum.IN_PROGRESS);
			if (synchronizedInfoOnly) {
				callBack.message(Messages.Tips_synchronizingDecryptInfo, 1);
			} else {
				callBack.message(Messages.Tips_startFocusDecryptSuccess, 1);
			}
		}else {
			callBack.message(Messages.Tips_startDecryptFail, -1);
		}
		callBack.finish(startRemoteDecrypt);
	
	}
}
