package com.gx.obe.opening.thread;

import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;

import com.gx.obe.WebserviceConstants;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.common.widget.listener.ProgressMessageListener;
import com.gx.obe.common.widget.thread.MyRunnable;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.DateUtils;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.SupplierService;
import com.gx.obe.web.service.TenderProjectService;
import com.gx.utils.ActionLogUtils;
import com.gx.utils.Params;

/** 
 * @Description: 批量远程解密
 * @author guoyr 
 */
public class BatchRemoteDecryptBidFilThread extends MyRunnable{
	
	private TenderProjectEntity tenderProjectEntity;
	private Date decryptEndTime;
	private String bidopeningStage;
	private ProgressMessageListener callBack;
	private List<TenderProjectEntity> tenderPackList;
	private Thread thread;
	private final String TIMESTAMP_STR = "yyyy-MM-dd HH:mm:ss";
	private SupplierService supplierService = new SupplierService();
	private Logger LOG = Logger.getLogger(BatchRemoteDecryptBidFilThread.class);
	private boolean synchronizedInfoOnly = false;
	private TenderProjectService tenderProjectService;
	public BatchRemoteDecryptBidFilThread(TenderProjectEntity tenderProjectEntity, Date decryptEndTime, String bidopeningStage, List<TenderProjectEntity> tenderPackList, boolean synchronizedInfoOnly, ProgressMessageListener callBack){
		this.tenderProjectEntity = tenderProjectEntity;
		this.decryptEndTime = decryptEndTime;
		this.bidopeningStage = bidopeningStage;
		this.synchronizedInfoOnly = synchronizedInfoOnly;
		this.callBack = callBack;
		this.tenderPackList = tenderPackList;
		this.tenderProjectService = new TenderProjectService();
	}
	public void start(){
		thread = new Thread(this);
		thread.start();
	}
	public void stop(){
		thread = null;
	}
	public void join(){
		if(null != thread){
			try {
				thread.join();
			} catch (InterruptedException e) {
			}
		}
	}
	public void run(){

		boolean startRemoteDecrypt = false;
		String openStageType = "00";
		boolean isPriceStage = false;
		boolean isBusAndTecStage = false;
		if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
			isPriceStage = true;
			openStageType = "02";
		}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
			openStageType = "01";
			isBusAndTecStage = true;
		}
	
		// 只同步解密信息
		if(synchronizedInfoOnly){
			startRemoteDecrypt = true;
			ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"批量同步远程解密信息", new Params("tenderNo", tenderProjectEntity.getTenderNo()).add("openStageType", openStageType));
		}else {
			callBack.message(Messages.Tips_starting, 0);
			HttpUrl httpUrl = HttpUtils.getProjectHttpUrl(tenderProjectEntity, PropertiesUtils.getWebserviceProperty(WebserviceConstants.START_REMOTE_DECRYPT_METHOD, ""));
			if(null != httpUrl){
				httpUrl.setName("批量启动远程解密");
				httpUrl.setAttribute("openStageType", openStageType);
				if(null != tenderPackList){
					StringBuilder stageIds = new StringBuilder();
					for(TenderProjectEntity tenderPack : tenderPackList){
						if(stageIds.length() > 0){
							stageIds.append(",");
						}
						stageIds.append(tenderPack.getStageId());
					}
					httpUrl.setAttribute("stageIds", stageIds.toString());
				}
				// 如果未设置解密截止时间，则为点击解密开始之后的1小时
				httpUrl.setAttribute("bidOpeningDeadLine", null != decryptEndTime ? DateUtils.getFormat(decryptEndTime, TIMESTAMP_STR) : "");// 解密截止时间
			}
			startRemoteDecrypt = HttpUtils.doPost(httpUrl).getStatus();
			ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"批量启动远程解密", httpUrl, startRemoteDecrypt);
			if(!startRemoteDecrypt) {
				startRemoteDecrypt = HttpUtils.doPost(httpUrl).getStatus();
				ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"第二次批量启动远程解密", httpUrl, startRemoteDecrypt);
			}
		}

		if(startRemoteDecrypt){
			for(TenderProjectEntity tenderPack : tenderPackList){
				// 初始化供应商开标结果
				supplierService.initSupplierBidOpeningResult(tenderPack.getId());
				if(null == tenderPack.getBidOpeningStatus() || !tenderPack.getBidOpeningStatus().equals(CommonEnum.COMPLETE)){
					tenderPack.setBidOpeningStatus(CommonEnum.IN_PROGRESS);
				}
				if(isPriceStage){
					tenderPack.setPriceBidStatus(CommonEnum.IN_PROGRESS);
				}else if(isBusAndTecStage){
					tenderPack.setBusinessBidStatus(CommonEnum.IN_PROGRESS);
					tenderPack.setTechnologyBidStatus(CommonEnum.IN_PROGRESS);
				}else{
					tenderPack.setBusinessBidStatus(CommonEnum.IN_PROGRESS);
					tenderPack.setTechnologyBidStatus(CommonEnum.IN_PROGRESS);
					tenderPack.setPriceBidStatus(CommonEnum.IN_PROGRESS);
				}
			}
			if(isPriceStage){
				tenderProjectService.batchUpdateProperty(tenderPackList, new String[]{"id","priceBidStatus"});
			}else if(isBusAndTecStage){
				tenderProjectService.batchUpdateProperty(tenderPackList, new String[]{"id","businessBidStatus", "technologyBidStatus", "bidOpeningStatus"});
			}else{
				tenderProjectService.batchUpdateProperty(tenderPackList, new String[]{"id","priceBidStatus", "businessBidStatus", "technologyBidStatus", "bidOpeningStatus"});
			}
			tenderProjectService.updateBidOpeningStatus(tenderProjectEntity, bidopeningStage, CommonEnum.IN_PROGRESS);
			if (synchronizedInfoOnly) {
				callBack.message(Messages.Tips_synchronizingDecryptInfo, 1);
			} else {
				callBack.message(Messages.Tips_startDecryptSuccess, 1);
			}
			
		}else {
			callBack.message(Messages.Tips_startDecryptFail, -1);
		}
		callBack.finish(startRemoteDecrypt);
	
	}
}
