package com.gx.obe.opening.thread;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.apache.log4j.Logger;
import org.dom4j.Document;

import com.gx.obe.WebserviceConstants;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.common.widget.thread.MyRunnable;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.opening.listener.BatchRemoteComplateAndUploadListener;
import com.gx.obe.opening.listener.BatchStopRemoteDecryptListener;
import com.gx.obe.opening.webservice.SupplierBidOpeningResulterCreater;
import com.gx.obe.web.entity.TransactionServer;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.TenderProjectService;
import com.gx.obe.web.vo.TenderPackOpeningInfo;
import com.gx.utils.ActionLogUtils;
import com.gx.utils.Params;

public class BatchUploadBidOpeningResultThread extends MyRunnable{
	
	private TenderProjectEntity tenderProjectEntity;
	private BatchRemoteComplateAndUploadListener listener;
	private Thread thread;
	private ThreadPoolExecutor threadPool;
	private List<TenderPackOpeningInfo> tenderPackList;
	private String bidopeningStage;
	private Logger LOG = Logger.getLogger(BatchUploadBidOpeningResultThread.class);
	private TenderProjectService tenderProjectService ;
	private boolean complateOnly = false;
	private String publicType;

	public BatchUploadBidOpeningResultThread(TenderProjectEntity tenderProjectEntity, List<TenderPackOpeningInfo> tenderPackList, String bidopeningStage, String publicType, BatchRemoteComplateAndUploadListener listener, boolean complateOnly){
		this.tenderProjectEntity = tenderProjectEntity;
		this.bidopeningStage = bidopeningStage;
		this.publicType = publicType;
		this.tenderPackList = tenderPackList;
		this.complateOnly = complateOnly;
		this.listener = listener;
		tenderProjectService = new TenderProjectService();

	}
	public void start(){
		thread = new Thread(this);
		thread.start();
	}
	public void stop(){
		thread = null;
		if(null != threadPool){
			threadPool.shutdownNow();
		}
	}
	public void join(){
		if(null != thread){
			try {
				thread.join();
			} catch (InterruptedException e) {
			}
		}
	}
	public void run(){
		final TransactionServer transactionServer = HttpUtils.getTransactionServer(tenderProjectEntity.getTransactionServerId());
		List<Future<TenderPackOpeningInfo>> allTenderPackFutureList = new ArrayList<Future<TenderPackOpeningInfo>>();
		List<TenderProjectEntity> allTenderPackList = tenderProjectService.getTenderPackBidOpeningInfoList(tenderProjectEntity.getId());
		if(null != allTenderPackList && allTenderPackList.size() > 0){
			
			
			List<TenderProjectEntity> noDecCompleteTenderPackList = new ArrayList<TenderProjectEntity>();
			// 是否需要在完成开标前先停止解密
			final boolean needStopOpeningFirst = PropertiesUtils.getProperty(ConfigConstants.NEED_STOP_BIDOPENING_FIRST, "00").equals("01");
			final List<String> uploadTenderIdList = new ArrayList<String>();
			if(null != tenderPackList){
				for(TenderPackOpeningInfo tenderPackInfo : tenderPackList){
					uploadTenderIdList.add(tenderPackInfo.getTenderProject().getId());
					if(!complateOnly){
						// 如果存在未解密完成的，并且存在解密家次少于签到家次的，则说明需要停止
						if(!tenderPackInfo.isDecComplate() || tenderPackInfo.getDecCount() < tenderPackInfo.getSignCount()){
							noDecCompleteTenderPackList.add(tenderPackInfo.getTenderProject());
						}
					}
				}
			}
			if(noDecCompleteTenderPackList.size() > 0){
				if(needStopOpeningFirst && !complateOnly){
					listener.showMessage(0, "正在请求停止远程解密...");
					// 先停止远程解密
					StopRemoteBidOpeningThread stopRemoteBidOpeningThread = new StopRemoteBidOpeningThread(tenderProjectEntity, noDecCompleteTenderPackList, bidopeningStage, transactionServer, new BatchStopRemoteDecryptListener(){
						public void stopRemoteDecrypt(boolean stopFlag,	String message) {
							listener.showMessage(stopFlag ? 1 : -1, message);
						}
					});
					stopRemoteBidOpeningThread.start();
					stopRemoteBidOpeningThread.join();
				}
				
				listener.showMessage(0, "正在同步最新解密信息...");
				// 这里最后再获取一次最新报价
				RemoteProjectDecryptInfoThread remoteProjectBidOpeningInfoThread = new RemoteProjectDecryptInfoThread(tenderProjectEntity, bidopeningStage, transactionServer, noDecCompleteTenderPackList, null);
				remoteProjectBidOpeningInfoThread.start();
				remoteProjectBidOpeningInfoThread.join();
				listener.showMessage(0, "已停止远程解密，正在同步最新解密信息！");
				
			}
			
			final boolean isPriceStage = (null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)) ? true : false;
			int successCount = 0;
			int failedCount = 0;
			
			int poolSize = allTenderPackList.size() > 5 ? 5 :allTenderPackList.size();
			threadPool = new ThreadPoolExecutor(poolSize, poolSize, 3, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>(),new ThreadPoolExecutor.CallerRunsPolicy());
			for(final TenderProjectEntity tenderPack : allTenderPackList){
				
				allTenderPackFutureList.add(threadPool.submit(new Callable<TenderPackOpeningInfo>() {
					public TenderPackOpeningInfo call() throws Exception {
						TenderPackOpeningInfo tenderPackOpeningInfo = null;
						
						if(uploadTenderIdList.contains(tenderPack.getId())){
							
							tenderPackOpeningInfo = uploadAndComplate(tenderPack, transactionServer);
							// 如果上传失败，再次进行上传
							if(null == tenderPackOpeningInfo){
								LOG.error(tenderPack.getTenderName() + "上传开标结果失败，进行第二次上传");
								tenderPackOpeningInfo = uploadAndComplate(tenderPack, transactionServer);
							}
							if(null != tenderPackOpeningInfo){
								listener.refreshUploadComplateInfo(tenderPackOpeningInfo);
							}
						}else {
							boolean isComplate = false;
							if(isPriceStage){
								isComplate = (null != tenderPack.getPriceBidStatus() && tenderPack.getPriceBidStatus().equals(CommonEnum.COMPLETE));
							}else {
								isComplate = (null != tenderPack.getBusinessBidStatus() && tenderPack.getBusinessBidStatus().equals(CommonEnum.COMPLETE));
							}
							if(isComplate){
								listener.refreshUploadComplateInfo(tenderProjectService.getTenderPackOpeningInfo(tenderPack.getId(), bidopeningStage));
							}
						}
						
						return tenderPackOpeningInfo;
					}
				}));
				
			}
			threadPool.shutdown();
			listener.showMessage(0, "正在请求完成解密，并上传结果...");
			for(Future<TenderPackOpeningInfo> future : allTenderPackFutureList){
				if(null == thread){
					break;
				}
				try {
					if(null != future.get()){
						successCount ++;
					}else {
						failedCount ++;
					}
				} catch (InterruptedException e) {
				} catch (ExecutionException e) {
				}finally{
					double percent = ((successCount + failedCount)/ (double)allTenderPackFutureList.size()) * (double)100;
					listener.showMessage(0, "已完成并上传" + (int)percent + "%" );
				}
			}
			noDecCompleteTenderPackList.clear();
			noDecCompleteTenderPackList = null;
			listener.finishComplateAndUpload(successCount, failedCount);
		}
	}
	
	
	/**
	 * @Description: 上传开标结果并完成开标
	 * @author guoyr
	 * @param tenderPack
	 * @param transactionServer
	 * @return 
	 */
	private TenderPackOpeningInfo uploadAndComplate(TenderProjectEntity tenderPack, TransactionServer transactionServer){
		TenderPackOpeningInfo tenderPackOpeningInfo = null;
		boolean uploadFlag = false;
		HttpUrl httpUrl = null;
		try {
			if(complateOnly){
				ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"未上传开标结果，仅完成开标", new Params("tenderNo", tenderPack.getTenderNo()));
				uploadFlag = true;
			}else {
				httpUrl = HttpUtils.getProjectHttpUrl(tenderPack, tenderProjectEntity, transactionServer, PropertiesUtils.getWebserviceProperty(WebserviceConstants.UPLOAD_SING_BIDRESULT_METHOD, ""));
				if(null != httpUrl){
					httpUrl.setName("批量上传开标结果");
					String openStageType = "00";
					if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
						openStageType = "02";
					}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
						openStageType = "01";
					}
					
					httpUrl.setAttribute("openStageType", openStageType);
					String bidOpeningResult = "";
					Document document = SupplierBidOpeningResulterCreater.createSupplierBidOpeningResultXml(tenderPack, bidopeningStage, false);
					if(null != document){
						bidOpeningResult = document.asXML();
					}
					httpUrl.setAttribute("bidOpeningResult", bidOpeningResult);
					httpUrl.setAttribute("publicType", publicType);
					uploadFlag = HttpUtils.doPost(httpUrl).getStatus();
				}
			}
		} catch (Exception e) {
			LOG.error("", e);
		}finally{
			ActionLogUtils.addServerLog(tenderPack.getId(),"批量上传开标结果", httpUrl, uploadFlag);
			if(uploadFlag){
				tenderProjectService.updateBidOpeningStatus(tenderPack, bidopeningStage, CommonEnum.COMPLETE);
				if(!complateOnly){
					updateUploadStatus(tenderPack);
				}
				tenderPackOpeningInfo = tenderProjectService.getTenderPackOpeningInfo(tenderPack.getId(), bidopeningStage);
//				listener.refreshUploadComplateInfo(tenderPackOpeningInfo);
			}else {
				listener.showMessage(-1, "["+tenderPack.getTenderNo() + tenderPack.getTenderName() +"]完成开标并上传结果失败！");
			}
		}
		return tenderPackOpeningInfo;
	}
	
	public void updateUploadStatus(TenderProjectEntity tenderPack){

		String[] params = null;
		if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
			tenderPack.setUploadPriceOpeningResultStatus(CommonEnum.COMPLETE);
			params = new String[]{"id","uploadPriceOpeningResultStatus"};
		}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
			tenderPack.setUploadOpeningResultStatus(CommonEnum.COMPLETE);
			params = new String[]{"id","uploadOpeningResultStatus"};
		}else {
			tenderPack.setUploadOpeningResultStatus(CommonEnum.COMPLETE);
			tenderPack.setUploadPriceOpeningResultStatus(CommonEnum.COMPLETE);
			params = new String[]{"id","uploadOpeningResultStatus", "uploadPriceOpeningResultStatus"};
		}
		tenderProjectService.updateAssignProperty(tenderPack, params);
	}
}
