package com.gx.obe.opening.thread;

import java.util.Date;
import java.util.List;
import java.util.concurrent.Future;

import org.apache.log4j.Logger;

import com.gx.obe.business.enumeration.SupplierEnum;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.listener.BidPriceResolveListener;
import com.gx.obe.opening.listener.DecryptBidFileProgressListener;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.SupplierOpeningResult;
import com.gx.obe.web.entity.price.BidPriceResult;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.SupplierOpeningResultService;

/**
 * @Description: 通过解密信封解密标书
 * @author guoyr
 */
public class DecryptBidFileRemoteThread implements IDecryptBidFileRunnable{
	
	private DecryptBidFileProgressListener decryptListener;
	private Logger LOG = Logger.getLogger(DecryptBidFileRemoteThread.class);
	private SupplierOpeningResultService supplierOpeningResultService ;
	private String supplierName = "";
	public static final String END_FLAG = "本次解密结束！";
	private BidPriceResolveListener bidPriceResolveListener;
	private TenderProjectEntity tenderProjectEntity;
	private Supplier supplier;
	private Thread thread;
	private String decryptStatus;
	private Date decryptTime;
	private boolean remoteDecryptComplate = false;
	
	public DecryptBidFileRemoteThread(DecryptBidFileParam decryptBidFileParam){
		this.tenderProjectEntity = decryptBidFileParam.tenderProjectEntity;
		this.supplier = decryptBidFileParam.supplier;
		this.decryptListener = decryptBidFileParam.decryptListener;
		this.bidPriceResolveListener = decryptBidFileParam.bidPriceResolveListener;
		supplierName = supplier.getSupplierName();
		supplierOpeningResultService = new SupplierOpeningResultService();
		
	}
	public void start(){
		thread = new Thread(this);
		thread.start();
	}
	public void stop(){
		thread = null;
	}
	public void join(){
		if(null != thread){
			try {
				thread.join();
			} catch (InterruptedException e) {
			}
		}
	}
	
	/**
	 * @Description: 远程解密完成
	 * @author guoyr
	 * @param decryptStatus
	 * @param decryptTime 
	 */
	public void remoteDecryptComplate(String decryptStatus, Date decryptTime){
		this.decryptStatus = decryptStatus;
		this.decryptTime = decryptTime;
		remoteDecryptComplate = true;
	}
	public void run() {
		String businessBidStatus = "";
		String technologyBidStatus = "";
		String priceBidStatus = "";
		decryptListener.start();
		try {
			decryptListener.info("正在等待远程解密...");
			while (null != thread){
				Thread.sleep(1000);
				if(remoteDecryptComplate){
					break;
				}
			}
			// 如果远程解密完成
			if(null != decryptTime){
				String bidOpeningStatus = "";
				if(null != decryptStatus && decryptStatus.equals("01")){
					bidOpeningStatus = SupplierEnum.OPENBID_SUCCESS;
				}else if(null != decryptStatus && decryptStatus.equals("02")){
					bidOpeningStatus = SupplierEnum.OPENBID_FIALE;
				}
				businessBidStatus = bidOpeningStatus;
				technologyBidStatus = bidOpeningStatus;
				priceBidStatus = bidOpeningStatus;
				technologyBidStatus = bidOpeningStatus;
				String openbidRecord = "";
				SupplierOpeningResult supplierOpeningResult = supplier.getSupplierOpeningResult();
				if(null == supplierOpeningResult || null == supplierOpeningResult.getId()){
					supplierOpeningResult = supplierOpeningResultService.queryOrCreateSupplierOpeningResult(tenderProjectEntity.getId(), supplier.getId());
				}
				if(bidOpeningStatus.equals(SupplierEnum.OPENBID_SUCCESS)){
					
					decryptListener.success("远程解密成功！");
					if(null != tenderProjectEntity.getPhaseType() && tenderProjectEntity.getPhaseType().equals(TenderProjectEnum.PHASE_TYPE_PRE)){
						// 资格预审项目不需要下载报价
						bidPriceResolveListener.saveSupplierBidPriceResult(supplierOpeningResult, businessBidStatus, technologyBidStatus, priceBidStatus, bidOpeningStatus, decryptTime, openbidRecord, null);
					}else {
						decryptListener.info("正在下载投标信息...");
						
						// 远程解密成功后下载投标报价
						Future<List<BidPriceResult>> future = bidPriceResolveListener.downloadRemoteBidOpeningResult(supplier);
						if(null != future.get()){
							// 下载投标报价后保存投标报价
							bidPriceResolveListener.saveSupplierBidPriceResult(supplierOpeningResult, businessBidStatus, technologyBidStatus, priceBidStatus, bidOpeningStatus, decryptTime, openbidRecord, future.get());
							decryptListener.success("投标信息下载成功！");
						}else {
							decryptListener.error("投标信息下载失败！");
						}
					}
				}else if(bidOpeningStatus.equals(SupplierEnum.OPENBID_FIALE)){
					bidPriceResolveListener.saveSupplierBidPriceResult(supplierOpeningResult, businessBidStatus, technologyBidStatus, priceBidStatus, bidOpeningStatus, decryptTime, openbidRecord, null);
					decryptListener.error("解密失败！");
				}
			}
		} catch (Exception e) {
			LOG.error("", e);
		}finally {
			decryptListener.after();
		}
	}

	public String getSupplierName() {
		return supplierName;
	}
	
	public String getOrgCode(){
		return supplier.getOrgCode();
	}
}
