package com.gx.obe.opening.thread;

import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.eclipse.swt.core.CallBack;

import com.gx.obe.business.enumeration.SupplierEnum;
import com.gx.obe.listener.BidPriceResolveListener;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.SupplierOpeningResult;
import com.gx.obe.web.entity.price.BidPriceResult;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.SupplierOpeningResultService;
import com.gx.obe.web.service.SupplierService;

public class DownloadSupplierBidPriceUtils {
	

	private TenderProjectEntity tenderProjectEntity;
	private BidPriceResolveListener bidPriceResolveListener;
	private ThreadPoolExecutor decryptBidFileThreadPool;
	private SupplierService supplierService = new SupplierService();
	private SupplierOpeningResultService supplierOpeningResultService ;
	
	public DownloadSupplierBidPriceUtils(TenderProjectEntity tenderProjectEntity){
		this.tenderProjectEntity = tenderProjectEntity;
		// 通过线程池解密文件
		int maxPoolSize = 5;
		// 初始化供应商开标结果
		supplierService.initSupplierBidOpeningResult(tenderProjectEntity.getId());
		bidPriceResolveListener = new BidPriceResolveListener(tenderProjectEntity, maxPoolSize);
		decryptBidFileThreadPool = new ThreadPoolExecutor(maxPoolSize, maxPoolSize, 3, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>());
	}
	
	public void downloadBidPrice(final Supplier supplier, final CallBack callBack){
		decryptBidFileThreadPool.submit(new Callable<SupplierOpeningResult>(){
			public SupplierOpeningResult call() throws Exception {
				List<BidPriceResult> bidPriceResultList = null;
				SupplierOpeningResult supplierOpeningResult = null;
				callBack.callBackString("  正在下载" + supplier.getSupplierName() + "报价！");
				// 远程解密成功后下载投标报价
				try {
					Future<List<BidPriceResult>> future = bidPriceResolveListener.downloadRemoteBidOpeningResult(supplier);
					bidPriceResultList = future.get();
				} catch (InterruptedException e) {
				} catch (ExecutionException e) {
				}finally{
					if(null != bidPriceResultList && bidPriceResultList.size() > 0){
						supplierOpeningResult = supplier.getSupplierOpeningResult();
						if(null == supplierOpeningResult || null == supplierOpeningResult.getId()){
							if(null == supplierOpeningResultService){
								supplierOpeningResultService = new SupplierOpeningResultService();
							}
							supplierOpeningResult = supplierOpeningResultService.queryOrCreateSupplierOpeningResult(tenderProjectEntity.getId(), supplier.getId());
						}
						supplierOpeningResult = bidPriceResolveListener.saveSupplierBidPriceResult(supplierOpeningResult, "", "", SupplierEnum.OPENBID_SUCCESS, SupplierEnum.OPENBID_SUCCESS, null, null, bidPriceResultList);
					}
					if(null != supplierOpeningResult){
						supplier.setSupplierOpeningResult(supplierOpeningResult);
						callBack.callBackBoolean(true);
						callBack.callBackString("√"+supplier.getSupplierName() + "报价下载成功！");
					}else {
						callBack.callBackBoolean(false);
						callBack.callBackString("×"+supplier.getSupplierName() + "报价下载失败！");
					}
					callBack.after();
				}
				return supplierOpeningResult;
			}
			
		});
		
	}
}
