package com.gx.obe.opening.thread;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import com.gx.obe.WebserviceConstants;
import com.gx.obe.business.http.HttpResponse;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.common.widget.listener.ProgressMessageListener;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.message.Messages;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.TransactionServer;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.SupplierService;
import com.gx.obe.web.service.TenderProjectService;
import com.gx.utils.ActionLogUtils;
import com.gx.utils.AnalyzeSupplierUtils;

/** 
 * @Description: 刷新项目下供应商的投标情况
 * @author guoyr 
 */
public class MultyProjectRefreshSupplierTenderInfoThread implements Runnable{

	private Thread thread;
	private TenderProjectService tenderProjectService ;
	private SupplierService supplierService = new SupplierService();
	private ProgressMessageListener messageListener;
	private List<TenderProjectEntity> tenderProjectList;
	private ThreadPoolExecutor threadPool;
	
	
	public MultyProjectRefreshSupplierTenderInfoThread(List<TenderProjectEntity> tenderProjectList, ProgressMessageListener callBack){
		this.tenderProjectList = tenderProjectList;
		this.messageListener = callBack;
		tenderProjectService = new TenderProjectService();
	}
	
	public void start(){
		thread = new Thread(this);
		thread.setName("在线同步投标情况");
		thread.start();
	}
	
	public void stop(){
		thread = null;
		if(null != threadPool){
			threadPool.shutdownNow();
		}
	}
	
	public void run() {
		if(null == tenderProjectList)return;
		int poolSize = tenderProjectList.size() > 5 ? 5 :tenderProjectList.size();
		threadPool = new ThreadPoolExecutor(poolSize, poolSize, 3, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>(),new ThreadPoolExecutor.CallerRunsPolicy());
		List<Future<Boolean>> allTenderPackFutureList = new ArrayList<Future<Boolean>>();
		messageListener.message("正在同步最新投标信息...", 0);
		for(final TenderProjectEntity tenderProjectEntity : tenderProjectList){
			TransactionServer transactionServer = HttpUtils.getTransactionServer(tenderProjectEntity.getTransactionServerId());
			String method = PropertiesUtils.getWebserviceProperty(WebserviceConstants.GET_PROJECT_SUPPLIER_METHOD, "");
			if(null == tenderProjectEntity.getParentId() && null != tenderProjectEntity.getIsPack() && tenderProjectEntity.getIsPack().equals(CommonEnum.YES)){
				List<TenderProjectEntity> tenderPackList = tenderProjectService.getSubPackageList(tenderProjectEntity.getId());
				if(null != tenderPackList){
					for(TenderProjectEntity tenderPack : tenderPackList){
						if(null != thread){
							allTenderPackFutureList.add(threadPool.submit(new Callable<Boolean>() {
								public Boolean call(){
									boolean flag = false;
									HttpUrl httpUrl = HttpUtils.getProjectHttpUrl(tenderPack, tenderProjectEntity, transactionServer, method);
									if(null != httpUrl){
										httpUrl.setName("同步供应商投标信息");
										messageListener.message("正在同步["+tenderPack.getTenderNo() +"]"+ tenderPack.getTenderName() + "投标信息...", 0);
										flag = resolveXml(tenderPack, HttpUtils.doPost(httpUrl));
										ActionLogUtils.addServerLog(tenderPack.getId(),"多项目批量同步供应商投标信息", httpUrl, flag);
									}
									return flag;
								}
							}));
							
						}
					}
				}
			}else {
				allTenderPackFutureList.add(threadPool.submit(new Callable<Boolean>() {
					public Boolean call(){
						HttpUrl httpUrl = HttpUtils.getProjectHttpUrl(tenderProjectEntity, null, transactionServer, method);
						boolean flag = false;
						if(null != httpUrl){
							httpUrl.setName("同步供应商投标信息");
							messageListener.message("正在同步["+tenderProjectEntity.getTenderNo() +"]"+ tenderProjectEntity.getTenderName() + "投标信息...", 0);
							flag = resolveXml(tenderProjectEntity, HttpUtils.doPost(httpUrl));
							ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"多项目批量同步供应商投标信息", httpUrl, flag);
						}
						return flag;
					}
				}));
			}
		
		}
		
		threadPool.shutdown();
		int synSuccessCount = 0;
		int synFaildCount = 0;
		for(Future<Boolean> future : allTenderPackFutureList){
			if(null == thread){
				break;
			}
			try {
				if(future.get()) {
					synSuccessCount ++;
				}else {
					synFaildCount ++;
				}
			} catch (InterruptedException e) {
			} catch (ExecutionException e) {
			}
		}
		if(null != thread && null != messageListener){
			if(synSuccessCount > 0 && synFaildCount == 0) {
				messageListener.message(Messages.Tips_synchronizTenderInfoComplate, 1);
			}else {
				messageListener.message(Messages.Tips_synchronizTenderInfoFail, -1);
			}
			messageListener.finish(synSuccessCount > 0 && synFaildCount == 0);
		}
	}
	
	/**
	 * @Description: 解析返回的xml
	 * @author guoyr
	 * @param xmlStr
	 * @return
	 */
	private boolean resolveXml(TenderProjectEntity tenderProjectEntity, HttpResponse httpResponse){
		List<Supplier> supplierList = null;
		
		if(null != httpResponse && httpResponse.getStatus()) {
			
			supplierList = AnalyzeSupplierUtils.analyzeJSON(tenderProjectEntity.getId(), httpResponse.getContent(), messageListener);
		}
		
		if(null == supplierList) {
			messageListener.message("["+tenderProjectEntity.getTenderNo() +"]"+ tenderProjectEntity.getTenderName() + "同步最新投标信息失败！", -1);
			return false;
		}
		if(supplierList.size() > 0) {
			int tenderCount = 0;
			for(Supplier supplier : supplierList) {
				if(null != supplier.getTenderTime()) {
					tenderCount ++;
				}
			}
			if (supplierService.batchSaveOrUpdate(supplierList, new String[]{"tenderTime", "orgId", "tenderFee", "bidBond", "isQualifiedSupplier", "modifyTime","isBasicAccount","inDate"}) > 0) {
				messageListener.message("["+tenderProjectEntity.getTenderNo() +"]"+ tenderProjectEntity.getTenderName() + "已同步最新投标信息，共投标["+tenderCount+"]家次！", 1);
				return true;
			}else {
				messageListener.message("["+tenderProjectEntity.getTenderNo() +"]"+ tenderProjectEntity.getTenderName() + "保存最新投标信息失败！", -1);
				return false;
			}
		}else {
			messageListener.message("["+tenderProjectEntity.getTenderNo() +"]"+ tenderProjectEntity.getTenderName() + "未同步到最新投标信息！", -1);
			return true;
		}
	}
}
