package com.gx.obe.opening.thread;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.gx.obe.common.widget.listener.ProgressMessageListener;
import com.gx.obe.common.widget.thread.MyRunnable;
import com.gx.obe.listener.ProgressMessageAdapter;
import com.gx.obe.message.Messages;
import com.gx.obe.opening.listener.StartRemoteBidOpeningListener;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.TenderProjectService;
import com.gx.obe.web.vo.TenderPackOpeningInfo;

/** 
 * @Description: 批量远程解密
 * @author guoyr 
 */
public class MultyProjectRemoteDecryptBidFilThread implements Runnable{
	
	private List<TenderProjectEntity> tenderProjectList;
	private String bidopeningStage;
	private StartRemoteBidOpeningListener startRemoteBidOpeningListener;
	private Date decryptEndTime;
	private ProgressMessageListener messageListener;
	private Thread thread;
	private boolean synchronizedInfoOnly = false;
	private List<MyRunnable> myRunnableList = new ArrayList<MyRunnable>();
	private int successCount = 0;
	private int failCount = 0;
	
	public MultyProjectRemoteDecryptBidFilThread(List<TenderProjectEntity> tenderProjectList, String bidopeningStage, Date decryptEndTime, boolean synchronizedInfoOnly, StartRemoteBidOpeningListener startRemoteBidOpeningListener, ProgressMessageListener callBack){
		this.tenderProjectList = tenderProjectList;
		this.bidopeningStage = bidopeningStage;
		this.synchronizedInfoOnly = synchronizedInfoOnly;
		this.startRemoteBidOpeningListener = startRemoteBidOpeningListener;
		this.decryptEndTime = decryptEndTime;
		this.messageListener = callBack;
	}
	public void start(){
		thread = new Thread(this);
		thread.start();
	}
	public void stop(){
		thread = null;
		for(MyRunnable myRunnable : myRunnableList){
			myRunnable.stop();
		}
	}
	public void join(){
		if(null != thread){
			try {
				thread.join();
			} catch (InterruptedException e) {
			}
		}
	}
	public void run(){
		messageListener.message(Messages.Tips_requesting, 0);
		TenderProjectService tenderProjectService = new TenderProjectService();
		for(final TenderProjectEntity tenderProjectEntity : tenderProjectList){
			if(null != tenderProjectEntity.getChildList() && tenderProjectEntity.getChildList().size() > 0){
				BatchRemoteDecryptBidFilThread batchRemoteDecryptBidFilThread = new BatchRemoteDecryptBidFilThread(tenderProjectEntity, decryptEndTime, bidopeningStage, tenderProjectEntity.getChildList(), synchronizedInfoOnly, new ProgressMessageAdapter() {
					public void finish(boolean flag) {
						count(flag);
						// 启动成功后查询最的开标状态
						TenderProjectEntity queryTenderProject = tenderProjectService.getById(tenderProjectEntity.getId());
						List<TenderPackOpeningInfo> tenderPackOpeningInfoList = tenderProjectService.getTenderPackOpeningInfoList(tenderProjectEntity.getId(), bidopeningStage);
						if(null != tenderPackOpeningInfoList && tenderPackOpeningInfoList.size() > 0){
							TenderPackOpeningInfo tenderProjectOpeningInfo = new TenderPackOpeningInfo(queryTenderProject, tenderProjectEntity.getChildList());
							for(TenderPackOpeningInfo tenderPackOpeningInfo : tenderPackOpeningInfoList){
								tenderProjectOpeningInfo.setTenderCount(tenderProjectOpeningInfo.getTenderCount() + tenderPackOpeningInfo.getTenderCount());
								tenderProjectOpeningInfo.setDecCount(tenderProjectOpeningInfo.getDecCount() + tenderPackOpeningInfo.getDecCount());
								tenderProjectOpeningInfo.setOpeningCount(tenderProjectOpeningInfo.getOpeningCount() + tenderPackOpeningInfo.getOpeningCount());
								tenderProjectOpeningInfo.setDecFailCount(tenderProjectOpeningInfo.getDecFailCount() + tenderPackOpeningInfo.getDecFailCount());
								tenderProjectOpeningInfo.setNoPriceCount(tenderProjectOpeningInfo.getNoPriceCount() + tenderPackOpeningInfo.getNoPriceCount());
								tenderProjectOpeningInfo.setComplateSupplierCount(tenderProjectOpeningInfo.getComplateSupplierCount() + tenderPackOpeningInfo.getComplateSupplierCount());
							}
							startRemoteBidOpeningListener.startRemoteBidOpening(flag, tenderProjectOpeningInfo, "");
						}
						showMessage(flag, tenderProjectEntity);
					}
				});
				batchRemoteDecryptBidFilThread.start();
				myRunnableList.add(batchRemoteDecryptBidFilThread);
			}else {
				// 启动远程解密
				StartRemoteDecryptBidFilThread startRemoteDecryptBidFilThread = new StartRemoteDecryptBidFilThread(tenderProjectEntity, decryptEndTime, bidopeningStage, new ProgressMessageAdapter() {
					public void finish(boolean flag) {
						count(flag);
						// 启动成功后查询最的开标状态
						startRemoteBidOpeningListener.startRemoteBidOpening(flag, tenderProjectService.getTenderPackOpeningInfo(tenderProjectEntity.getId(), bidopeningStage), "");
						showMessage(flag, tenderProjectEntity);
					}
				}, synchronizedInfoOnly);
				startRemoteDecryptBidFilThread.start();
				myRunnableList.add(startRemoteDecryptBidFilThread);
			}
		}
		for(MyRunnable myRunnable : myRunnableList){
			myRunnable.join();
		}
		messageListener.finish(successCount > 0 && failCount == 0);
	}
	
	private synchronized void count(boolean flag) {
		if(flag) {
			successCount ++;
		}else {
			failCount ++;
		}
	}
	private void showMessage(boolean flag, TenderProjectEntity tenderProjectEntity) {
		if(flag) {
			if(synchronizedInfoOnly){
				messageListener.message(MessageFormat.format(Messages.Tips_synchronizingProjectDecryptInfo, tenderProjectEntity.getTenderName()), 1);
			}else {
				messageListener.message(MessageFormat.format(Messages.Tips_startProjectDecryptSuccess, tenderProjectEntity.getTenderName()), 1);
			}
		}else {
			messageListener.message(MessageFormat.format(Messages.Tips_startProjectDecryptFail, tenderProjectEntity.getTenderName()), -1);
		}
	}
}
