package com.gx.obe.opening.thread;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import com.gx.obe.common.widget.thread.MyRunnable;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.listener.ProgressMessageAdapter;
import com.gx.obe.opening.listener.BatchRemoteComplateAndUploadListener;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.TenderProjectService;
import com.gx.obe.web.vo.TenderPackOpeningInfo;

public class MultyProjectUploadBidOpeningResultThread implements Runnable{

	
	private BatchRemoteComplateAndUploadListener listener;
	private Thread thread;
	private ThreadPoolExecutor threadPool;
	private List<TenderPackOpeningInfo> projectOpeningInfoList;
	private String bidopeningStage;
	private String publicType;
	private List<MyRunnable> myRunnableList = new ArrayList<MyRunnable>();
	private TenderProjectService tenderProjectService;
	private boolean complateOnly = false;
	
	public MultyProjectUploadBidOpeningResultThread(List<TenderPackOpeningInfo> projectOpeningInfoList, String bidopeningStage, String publicType, BatchRemoteComplateAndUploadListener listener, boolean complateOnly){
		this.projectOpeningInfoList = projectOpeningInfoList;
		this.bidopeningStage = bidopeningStage;
		this.publicType = publicType;
		this.complateOnly = complateOnly;
		this.listener = listener;
		this.tenderProjectService = new TenderProjectService();
	}
	
	
	public void start(){
		thread = new Thread(this);
		thread.setName(projectOpeningInfoList.size()+"个项目完成开标" + bidopeningStage);
		thread.start();
	}
	
	public void stop(){
		thread = null;
		for(MyRunnable myRunnable : myRunnableList){
			myRunnable.stop();
		}
		if(null != threadPool){
			threadPool.shutdownNow();
		}
	}
	
	public void run() {
		final boolean isPriceStage = (null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)) ? true : false;
		int poolSize = 1;//tenderProjectList.size() > 5 ? 5 :tenderProjectList.size();
		threadPool = new ThreadPoolExecutor(poolSize, poolSize, 3, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>(),new ThreadPoolExecutor.CallerRunsPolicy());
		List<Future<TenderPackOpeningInfo>> allTenderPackFutureList = new ArrayList<Future<TenderPackOpeningInfo>>();
		
		for(final TenderPackOpeningInfo projectOpeningInfo : projectOpeningInfoList){
			if(null != projectOpeningInfo.getTenderPackOpeningInfoList() && projectOpeningInfo.getTenderPackOpeningInfoList().size() > 0){
				
				allTenderPackFutureList.add(threadPool.submit(new Callable<TenderPackOpeningInfo>() {
					public TenderPackOpeningInfo call() throws Exception {
						
						// 按项目批量获取解密信息
						BatchUploadBidOpeningResultThread uploadBidOpeningResultThread = new BatchUploadBidOpeningResultThread(projectOpeningInfo.getTenderProject(), projectOpeningInfo.getTenderPackOpeningInfoList(), bidopeningStage, publicType, new BatchRemoteComplateAndUploadListener(){
							
							Map<String, TenderPackOpeningInfo> tenderProjectOpeningInfoMap = new HashMap<String, TenderPackOpeningInfo>();
							public synchronized void refreshUploadComplateInfo(TenderPackOpeningInfo _tenderPackOpeningInfo) {
								TenderProjectEntity _tenderPack = _tenderPackOpeningInfo.getTenderProject();
								tenderProjectOpeningInfoMap.put(_tenderPackOpeningInfo.getTenderProject().getId(), _tenderPackOpeningInfo);
								boolean isComplate = false;
								if(isPriceStage){
									isComplate = (null != _tenderPack.getPriceBidStatus() && _tenderPack.getPriceBidStatus().equals(CommonEnum.COMPLETE));
								}else {
									isComplate = (null != _tenderPack.getBusinessBidStatus() && _tenderPack.getBusinessBidStatus().equals(CommonEnum.COMPLETE));
								}
								if(isComplate){
									
									int complatePackCount = 0;
									int complateSupplierCount = 0;
									for(Entry<String, TenderPackOpeningInfo> entry : tenderProjectOpeningInfoMap.entrySet()){
										TenderPackOpeningInfo tenderPackOpeningInfo = entry.getValue();
										if(tenderPackOpeningInfo.isComplate()){
											
											complatePackCount += 1;
											complateSupplierCount += tenderPackOpeningInfo.getDecCount();
										}
									}
									if(complateSupplierCount > projectOpeningInfo.getComplateSupplierCount()){
										projectOpeningInfo.setComplatePackCount(complatePackCount);
										projectOpeningInfo.setComplateSupplierCount(complateSupplierCount);
										listener.refreshUploadComplateInfo(projectOpeningInfo);
									}
								}
							}
							public synchronized void finishComplateAndUpload(int successCount, int failCount) {
							}

							public void showMessage(int flag, String message) {
								if(flag < 0) {
									listener.showMessage(0, "[" + projectOpeningInfo.getTenderProject().getTenderName()+"]完成开标并上传结果失败！");
								}
								
							}
						}, complateOnly);
						uploadBidOpeningResultThread.start();
						listener.showMessage(0, "[" + projectOpeningInfo.getTenderProject().getTenderName()+"]正在完成并上传结果...");
						myRunnableList.add(uploadBidOpeningResultThread);
						uploadBidOpeningResultThread.join();
						return projectOpeningInfo;
					}
				}));
				
			}else {
				allTenderPackFutureList.add(threadPool.submit(new Callable<TenderPackOpeningInfo>() {
					public TenderPackOpeningInfo call() throws Exception {
						if(complateOnly){
							tenderProjectService.updateBidOpeningStatus(projectOpeningInfo.getTenderProject(), bidopeningStage, CommonEnum.COMPLETE);
							listener.refreshUploadComplateInfo(tenderProjectService.getTenderPackOpeningInfo(projectOpeningInfo.getTenderProject().getId(), bidopeningStage));
						}else {
							UploadBidOpeningResultThread uploadBidOpeningResultThread = new UploadBidOpeningResultThread(projectOpeningInfo.getTenderProject(), bidopeningStage, publicType, new ProgressMessageAdapter(){
								public void finish(final boolean flag) {
									if(flag){
										TenderPackOpeningInfo tenderPackOpeningInfo = tenderProjectService.getTenderPackOpeningInfo(projectOpeningInfo.getTenderProject().getId(), bidopeningStage);
										listener.refreshUploadComplateInfo(tenderPackOpeningInfo);
									}else {
										listener.showMessage(-1, "["+projectOpeningInfo.getTenderProject().getTenderNo() + projectOpeningInfo.getTenderProject().getTenderName() +"]完成开标并上传结果失败！");
									}
								}
							});
							uploadBidOpeningResultThread.start();
							listener.showMessage(0, "[" + projectOpeningInfo.getTenderProject().getTenderName() + "]正在完成并上传结果...");
							myRunnableList.add(uploadBidOpeningResultThread);
							uploadBidOpeningResultThread.join();
						}
						return projectOpeningInfo;
					}
				}));
			}
		}
		
		for(Future<TenderPackOpeningInfo> future : allTenderPackFutureList){
			if(null == thread){
				break;
			}
			try {
				future.get();
			} catch (InterruptedException e) {
			} catch (ExecutionException e) {
			}
		}
		listener.finishComplateAndUpload(0, 0);
	}
	
}
