package com.gx.obe.opening.thread;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.apache.log4j.Logger;

import com.gx.obe.WebserviceConstants;
import com.gx.obe.business.enumeration.SupplierEnum;
import com.gx.obe.business.http.HttpResponse;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.common.widget.thread.MyRunnable;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.listener.BidPriceResolveListener;
import com.gx.obe.opening.listener.DecryptRemoteBidFileListener;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.SupplierOpeningResult;
import com.gx.obe.web.entity.price.BidPriceResult;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.SupplierOpeningResultService;
import com.gx.obe.web.service.SupplierService;
import com.gx.utils.ActionLogUtils;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

/** 
 * @Description: 获得供应商远程解密状态
 * @author guoyr 
 */
public class RemoteDecryptStatusThread extends MyRunnable{
	private TenderProjectEntity tenderProjectEntity;
	private String bidopeningStage;
	private Thread thread;
	private Logger LOG = Logger.getLogger(RemoteDecryptStatusThread.class);
	private SupplierService supplierService = new SupplierService();
	private boolean isPriceStage = false;
	private boolean isBusAndTecStage = false;
	private Map<String, Supplier> decryptStatusMap;
	private DecryptRemoteBidFileListener listener;
	private int totalSupplierCount = 0;
	private int totalNoDecCount = 0;
	private int totalNoPriceCount = 0;
	private int totalBidSuccessCount = 0;
	private int totalBidFailCount = 0;
	private BidPriceResolveListener bidPriceResolveListener;
	private List<String> resolverSupplierIdList;
	private List<String> bidOpeningComplateSupplierIdList;
	private ThreadPoolExecutor decryptBidFileThreadPool;
	private boolean onlyOne = false;
	
	
	public RemoteDecryptStatusThread(TenderProjectEntity tenderProjectEntity, String bidopeningStage, DecryptRemoteBidFileListener listener){
		this(tenderProjectEntity, bidopeningStage, listener, false);
	}
	public RemoteDecryptStatusThread(TenderProjectEntity tenderProjectEntity, String bidopeningStage, DecryptRemoteBidFileListener listener, boolean onlyOne){
		this.tenderProjectEntity = tenderProjectEntity;
		this.bidopeningStage = bidopeningStage;
		this.listener = listener;
		this.onlyOne = onlyOne;
	}
	
	public void start(){
		thread = new Thread(this);
		thread.start();
	}
	public void stop(){
		if(null != decryptBidFileThreadPool){
			decryptBidFileThreadPool.shutdownNow();
		}
		if(null != bidOpeningComplateSupplierIdList){
			bidOpeningComplateSupplierIdList.clear();
		}
		if(null != decryptStatusMap){
			decryptStatusMap.clear();
		}
		thread = null;
	}
	public void join(){
		if(null != thread){
			try {
				thread.join();
			} catch (InterruptedException e) {
			}
		}
	}
	
	public synchronized void refreshBidOpeningStatus(Supplier supplier, SupplierOpeningResult supplierOpeningResult, boolean hasBidPrice, boolean rememberMe){
		if(null != thread){
			if(!bidOpeningComplateSupplierIdList.contains(supplier.getId())){
				if(rememberMe){
					bidOpeningComplateSupplierIdList.add(supplier.getId());
				}
				
				if(null != listener){
					listener.refreshBidOpeningStatus(supplier, supplierOpeningResult.getBusinessBidStatus(), supplierOpeningResult.getTechnologyBidStatus(), supplierOpeningResult.getPriceBidStatus(), hasBidPrice);
				}
			}
		}
	}
	
	public void run(){
		HttpUrl httpUrl = null;
		try {
			httpUrl = HttpUtils.getProjectHttpUrl(tenderProjectEntity, PropertiesUtils.getWebserviceProperty(WebserviceConstants.GET_SUPPLIER_DECRYPT_STAUTS_METHOD, "bidOpeningHS.do?getSupplierBidOpeningDecryptStauts"));
			if(null != httpUrl){
				httpUrl.setName("获得远程开标信息");
				// openStageType : [00一次开标阶段，01非价格开标阶段，02价格开标阶段]
				String openStageType = "00";
				if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
					isPriceStage = true;
					openStageType = "02";
				}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
					isBusAndTecStage = true;
					openStageType = "01";
				}
				httpUrl.setAttribute("openStageType", openStageType);
				
				resolverSupplierIdList = new ArrayList<String>();
				bidOpeningComplateSupplierIdList = new ArrayList<String>();
				supplierService.initSupplierBidOpeningResult(tenderProjectEntity.getId());
				
				List<Supplier> supplierList = supplierService.getSupplierOpenBidList(tenderProjectEntity.getId());
				// 通过线程池解密文件
				int maxPoolSize = null != supplierList && supplierList.size() < 5 ? supplierList.size() : 5;
				bidPriceResolveListener = new BidPriceResolveListener(tenderProjectEntity, maxPoolSize);
				decryptBidFileThreadPool = new ThreadPoolExecutor(maxPoolSize, maxPoolSize, 3, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>());
				decryptStatusMap = new HashMap<String, Supplier>();
				boolean isPreEvaluation = false;
				if(null != tenderProjectEntity.getPhaseType() && tenderProjectEntity.getPhaseType().equals(TenderProjectEnum.PHASE_TYPE_PRE)){
					isPreEvaluation = true;
				}
				long startTime = 0;
				long useTime = -1;
				while(null != thread){
					startTime = Calendar.getInstance().getTimeInMillis();
					totalNoPriceCount = 0;
					totalBidFailCount = 0;
					totalBidSuccessCount = 0;
					totalSupplierCount = 0;
					totalNoDecCount = 0;
					LOG.debug("获取供应商远程解密状态中...");
					if(null != supplierList && supplierList.size() > 0){
						Set<String> supplierOrgCodeSet = new HashSet<String>();
						for(Supplier supplier : supplierList){
							if(null == thread){
								break;
							}
							if(null == supplier.getTenderTime()){
								continue;
							}
							SupplierOpeningResult supplierOpeningResult = supplier.getSupplierOpeningResult();
							String decsFlag = "";
							boolean hasBidPrice = false;
							if(null != supplierOpeningResult && null != supplierOpeningResult.getId()){
								if(null != supplierOpeningResult.getBidPrice() && supplierOpeningResult.getBidPrice().compareTo(BigDecimal.ZERO) > 0){
									hasBidPrice = true;
								}
								// 判断投标人是否废标
								if(null != supplierOpeningResult.getRevokeStatus() && supplierOpeningResult.getRevokeStatus().equals(SupplierEnum.REVOKE_YES)){
									refreshBidOpeningStatus(supplier, supplierOpeningResult, hasBidPrice, true);
									continue;
								}
								if(isPriceStage){
									if(null != supplierOpeningResult.getPriceBidStatus() && supplierOpeningResult.getPriceBidStatus().equals(SupplierEnum.OPENBID_SUCCESS)){
										decsFlag = SupplierEnum.OPENBID_SUCCESS;
									}else if(null != supplierOpeningResult.getPriceBidStatus() && supplierOpeningResult.getPriceBidStatus().equals(SupplierEnum.OPENBID_FIALE)){
										decsFlag = SupplierEnum.OPENBID_FIALE;
									}
								}else if(isBusAndTecStage) {
									if(null != supplierOpeningResult.getBusinessBidStatus() && supplierOpeningResult.getBusinessBidStatus().equals(SupplierEnum.OPENBID_SUCCESS) && null != supplierOpeningResult.getTechnologyBidStatus() && supplierOpeningResult.getTechnologyBidStatus().equals(SupplierEnum.OPENBID_SUCCESS)){
										decsFlag = SupplierEnum.OPENBID_SUCCESS;
									}else if((null != supplierOpeningResult.getBusinessBidStatus() && supplierOpeningResult.getBusinessBidStatus().equals(SupplierEnum.OPENBID_FIALE)) || (null != supplierOpeningResult.getTechnologyBidStatus() && supplierOpeningResult.getTechnologyBidStatus().equals(SupplierEnum.OPENBID_FIALE))){
										decsFlag = SupplierEnum.OPENBID_FIALE;
									}
								}else {
									if(null != supplierOpeningResult.getBusinessBidStatus() && supplierOpeningResult.getBusinessBidStatus().equals(SupplierEnum.OPENBID_SUCCESS) && null != supplierOpeningResult.getTechnologyBidStatus() && supplierOpeningResult.getTechnologyBidStatus().equals(SupplierEnum.OPENBID_SUCCESS)){
										decsFlag = SupplierEnum.OPENBID_SUCCESS;
									}else if((null != supplierOpeningResult.getBusinessBidStatus() && supplierOpeningResult.getBusinessBidStatus().equals(SupplierEnum.OPENBID_FIALE)) || (null != supplierOpeningResult.getTechnologyBidStatus() && supplierOpeningResult.getTechnologyBidStatus().equals(SupplierEnum.OPENBID_FIALE))){
										decsFlag = SupplierEnum.OPENBID_FIALE;
									}
								}
							}else{
								supplierOpeningResult = new SupplierOpeningResult();
								supplierOpeningResult.setTenderId(tenderProjectEntity.getId());
								supplierOpeningResult.setSupplierId(supplier.getId());
								supplier.setSupplierOpeningResult(supplierOpeningResult);
							}
							totalSupplierCount ++;
							if(decsFlag.equals(SupplierEnum.OPENBID_SUCCESS)){
								boolean isRealComplate = true;
								totalBidSuccessCount ++;
								if(!isBusAndTecStage){
									// 如果是非资审项目，并且没有报价，则标记为错
									if(!isPreEvaluation && !hasBidPrice){
										totalNoPriceCount ++;
										isRealComplate = false;// 如果开标成功，却没有报价，则下次继续刷新
									}
								}
								refreshBidOpeningStatus(supplier, supplierOpeningResult, hasBidPrice, isRealComplate);
								
								if(isRealComplate){
//									if(!onlyOne){
										// 如果已经开标成功了，则刷完后不再读取开标结果
									continue;
								}
							}else if(decsFlag.equals(SupplierEnum.OPENBID_FIALE)){
								totalBidFailCount ++;
								boolean isRealComplate = false;
								if(isPriceStage){
									// 如果已经确认了解密失败，则记住，不需要重复刷新
									if(null != supplierOpeningResult.getPriceConfirmStatus() && supplierOpeningResult.getPriceConfirmStatus().length() > 0){
										isRealComplate = true;
									}
								}else {
									if(null != supplierOpeningResult.getConfirmStatus() && supplierOpeningResult.getConfirmStatus().length() > 0){
										isRealComplate = true;
									}
								}
								refreshBidOpeningStatus(supplier, supplierOpeningResult, hasBidPrice, isRealComplate);
							}else {
								totalNoDecCount ++;
							}
							supplierOrgCodeSet.add(supplier.getOrgCode());
							decryptStatusMap.put(supplier.getOrgCode().trim(), supplier);
						}
						
						if(null != listener){
							listener.refreshProgress(totalSupplierCount, totalBidSuccessCount, totalNoDecCount, totalBidFailCount, totalNoPriceCount);
						}
						// 如果存在需要查询解密状态的供应商则调用接口
						if(supplierOrgCodeSet.size() > 0){

							// 每隔10秒刷新
							if(useTime > 5000){
								// 此次刷新时间超过5秒
							}else if(useTime > 0){
								try {
									Thread.sleep(5000 - useTime);
								} catch (InterruptedException e) {
								}
							}
							resolveXml(HttpUtils.doPost(httpUrl), supplierOrgCodeSet);
							if(totalNoPriceCount > 0 && totalNoPriceCount == totalBidSuccessCount && totalNoPriceCount == totalSupplierCount) {
								// 所有都没有报价，则完成解密
								break;
							}
						}else {
							break;
						}
					}
					useTime = Calendar.getInstance().getTimeInMillis() - startTime;
					// 全部开标完成
					if(totalBidFailCount + totalNoPriceCount == 0 && totalBidSuccessCount >= totalSupplierCount){
//					if(totalBidSuccessCount + totalBidFaildCount >= totalSupplierCount){
						break;
					}
					if(onlyOne){
						break;
					}
					supplierList = supplierService.getSupplierOpenBidList(tenderProjectEntity.getId());
				}
			}
		} catch (Exception e) {
			stop();
		}finally{
			if(null != thread) {
				ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"当前可解密标书已全部解密完成", httpUrl, true);
			}else {
				ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"停止获得远程开标信息", httpUrl, true);
			}
			LOG.debug("获取供应商远程解密状态结束！");
			if(null != thread && null != listener){
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
				}
				listener.stopRemoteDecrypt();
			}
			stop();
		}
	}
	
	/**
	 * @Description: 解析服务器数据
	 * @author guoyr
	 * @param xmlStr
	 * @param supplierOrgCodeSet 只需要检查更新的供应商机构集合
	 */
	private void resolveXml(HttpResponse httpResponse, Set<String> supplierOrgCodeSet){
		try {
			if(null != httpResponse && httpResponse.getStatus()){
				SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
				JSONArray supplierListArray = httpResponse.getDataJSONArray("supplierBidOpeningStatusList");
				if(null == supplierListArray){
					return;
				}
				Object[] supplierJsonList =  supplierListArray.toArray();
				if(null != supplierJsonList && supplierJsonList.length > 0){
					List<Future<SupplierOpeningResult>> allSupplierOpeningResultFutureList = new ArrayList<Future<SupplierOpeningResult>>();
					
					for(Object object : supplierJsonList){
						JSONObject supplierJson = (JSONObject)object;
						String orgCode = (String)supplierJson.get("orgCode");
						//过滤只有远程解密的供应商进入要更新列表
						if(supplierOrgCodeSet.contains(orgCode)){
							String decryptStatus = (String)supplierJson.get("bidOpeningStatus");
							Date bidOpeningTime = null;
							try {
								if(null != supplierJson.get("bidOpeningTime") && !"".equals(supplierJson.get("bidOpeningTime"))){
									bidOpeningTime = format.parse((String)supplierJson.get("bidOpeningTime"));//解密状态
								}
							} catch (ParseException e) {
								LOG.error("时间格式化错误："+ e.getMessage());
							}
							// 如果有远程解密时间和远程解状状态，则说明远程解密完成
							if(null != decryptStatus && decryptStatus.length() > 0 && null != bidOpeningTime){
								final Supplier supplier = decryptStatusMap.get(orgCode);
								
								if(null != supplier){
									final SupplierOpeningResult supplierOpeningResult = supplier.getSupplierOpeningResult();
									boolean dencFlag = false;
									String bidOpeningStatus = null;
									if(null != decryptStatus && decryptStatus.equals("01")){
										bidOpeningStatus = SupplierEnum.OPENBID_SUCCESS;
										dencFlag = true;
									}
									if(isPriceStage){
										supplierOpeningResult.setPriceBidStatus(bidOpeningStatus);
									}else if(isBusAndTecStage){
										supplierOpeningResult.setBusinessBidStatus(bidOpeningStatus);
										supplierOpeningResult.setTechnologyBidStatus(bidOpeningStatus);
										
									}else {
										supplierOpeningResult.setBusinessBidStatus(bidOpeningStatus);
										supplierOpeningResult.setTechnologyBidStatus(bidOpeningStatus);
										supplierOpeningResult.setPriceBidStatus(bidOpeningStatus);
									}
									
									final String supplierId = supplierOpeningResult.getSupplierId();
									if(addOpeningSupplierId(supplierId)){
										final boolean isDecFlag = dencFlag;
										final Date decTime = bidOpeningTime;
										allSupplierOpeningResultFutureList.add(decryptBidFileThreadPool.submit(new Callable<SupplierOpeningResult>(){
											public SupplierOpeningResult call() throws Exception {
												try {
													return createSupplierBidOpeningResult(supplier, isDecFlag, "", decTime);
												} catch (Exception e) {
													LOG.error("", e);
												}finally{
													removeOpeningSupplierId(supplierId);
												}
												return supplierOpeningResult;
											}
											
										}));
									}
								}
							}
						}
					}
					if(allSupplierOpeningResultFutureList.size() > 0){
						List<SupplierOpeningResult> updateSupplierOpeningResultList = new ArrayList<SupplierOpeningResult>();
						
						for(Future<SupplierOpeningResult> future : allSupplierOpeningResultFutureList){
							if(null == thread){
								break;
							}
							try {
								updateSupplierOpeningResultList.add(future.get());
							} catch (InterruptedException e) {
							} catch (ExecutionException e) {
							}
						}
					}
					
				}
			
			}
		} catch (Exception e) {
			LOG.error("返回的xml结果为："+httpResponse.getContent(), e);
		}finally{
		}
	}
	
	/**
	 * @Description: 生成开标记录
	 * @author guoyr
	 * @param decEnvelopDocument
	 * @param bidFilePath
	 * @param tenderId
	 * @param supplierId
	 * @param bidOpeningStatus
	 * @param openbidRecord 
	 */
	private SupplierOpeningResult createSupplierBidOpeningResult(Supplier supplier, boolean dencFlag, String openbidRecord, Date decryptTime){
		
		List<BidPriceResult> bidPriceResultList = null;
		
		String businessBidStatus = "";
		String technologyBidStatus = "";
		String priceBidStatus = "";
		
		boolean hasPrice = false;
		String decStatus = dencFlag ? SupplierEnum.OPENBID_SUCCESS : SupplierEnum.OPENBID_FIALE;
		if(isPriceStage){
			hasPrice = true;
			priceBidStatus = decStatus;
		}else if(isBusAndTecStage){
			businessBidStatus = decStatus;
			technologyBidStatus = decStatus;
		}else {
			hasPrice = true;
			businessBidStatus = decStatus;
			technologyBidStatus = decStatus;
			priceBidStatus = decStatus;
		}
		boolean isPreEvaluation = false;
		if(null != tenderProjectEntity.getPhaseType() && tenderProjectEntity.getPhaseType().equals(TenderProjectEnum.PHASE_TYPE_PRE)){
			isPreEvaluation = true;
		}
		if(hasPrice && dencFlag && !isPreEvaluation){
			// 远程解密成功后下载投标报价
			Future<List<BidPriceResult>> future = bidPriceResolveListener.downloadRemoteBidOpeningResult(supplier);
			try {
				bidPriceResultList = future.get();
			} catch (InterruptedException e) {
			} catch (ExecutionException e) {
			}finally{
				if(null == bidPriceResultList || bidPriceResultList.size() == 0){
					LOG.error(supplier.getSupplierName() + "未下载到报价信息");
				}
			}
		}
		if(null != bidPriceResolveListener){
			
			SupplierOpeningResult supplierOpeningResult = supplier.getSupplierOpeningResult();
			if(null == supplierOpeningResult || null == supplierOpeningResult.getId()){
				LOG.error(supplier.getSupplierName() + "开标结果有误");
				SupplierOpeningResultService supplierOpeningResultService = new SupplierOpeningResultService();
				supplierOpeningResult = supplierOpeningResultService.queryOrCreateSupplierOpeningResult(tenderProjectEntity.getId(), supplier.getId());
			}
			return bidPriceResolveListener.saveSupplierBidPriceResult(supplierOpeningResult, businessBidStatus, technologyBidStatus, priceBidStatus, dencFlag?SupplierEnum.OPENBID_SUCCESS:SupplierEnum.OPENBID_FIALE, decryptTime, openbidRecord, bidPriceResultList);
		}
		return null;
	}
	
	/**
	 * @Description: 添加正在开标中的供应商
	 * @author guoyr
	 * @param supplierId
	 * @return 
	 */
	public synchronized boolean addOpeningSupplierId(String supplierId){
		if(null != resolverSupplierIdList && !resolverSupplierIdList.contains(supplierId)){
			resolverSupplierIdList.add(supplierId);
			return true;
		}
		return false;
	}
	/**
	 * @Description: 删除正在开标中的供应商
	 * @author guoyr
	 * @param supplierId
	 * @return 
	 */
	public synchronized void removeOpeningSupplierId(String supplierId){
		if(null != resolverSupplierIdList){
			resolverSupplierIdList.remove(supplierId);
		}
	}
	
	
	public static interface RemoteDecryptListener{
		public void refreshDecryptStatus(Map<String, Supplier> signInfoMap);
	}
}
