package com.gx.obe.opening.thread;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.apache.log4j.Logger;

import com.gx.obe.WebserviceConstants;
import com.gx.obe.business.enumeration.SupplierEnum;
import com.gx.obe.business.http.HttpResponse;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.listener.BidPriceResolveListener;
import com.gx.obe.opening.listener.BatchRemoteDecryptListener;
import com.gx.obe.util.utils.DateUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.SupplierOpeningResult;
import com.gx.obe.web.entity.TransactionServer;
import com.gx.obe.web.entity.price.BidPriceResult;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.SupplierOpeningResultService;
import com.gx.obe.web.service.SupplierService;
import com.gx.obe.web.service.TenderProjectService;
import com.gx.utils.ActionLogUtils;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

/**
 * @Description: 获得项目下的包件的开标信息（解密和确认）
 * @author guoyr
 */
public class RemoteProjectDecryptInfoThread implements Runnable{
	
	private Logger LOG = Logger.getLogger(RemoteProjectDecryptInfoThread.class);
	private TenderProjectEntity tenderProjectEntity;
	private TransactionServer transactionServer;
	private List<TenderProjectEntity> tenderPackList;
	private String bidopeningStage;
	private Thread thread;
	private SupplierService supplierService = new SupplierService();
	private SupplierOpeningResultService supplierOpeningResultService ;
	private TenderProjectService tenderProjectService;
	private boolean isPriceStage = false;
	private boolean isBusAndTecStage = false;
 	private BatchRemoteDecryptListener remoteProjectBidOpeningListener;
	private ThreadPoolExecutor downloadDecInfoThreadPool;
	private List<ThreadPoolExecutor> downloadBidPriceThreadPoolList;
	private int maxThreadPoolCount = 5;
	
	public RemoteProjectDecryptInfoThread(TenderProjectEntity tenderProjectEntity, String bidopeningStage, TransactionServer transactionServer, List<TenderProjectEntity> tenderPackList, BatchRemoteDecryptListener remoteProjectBidOpeningListener){
		this.tenderProjectEntity = tenderProjectEntity;
		this.bidopeningStage = bidopeningStage;
		this.transactionServer = transactionServer;
		this.tenderPackList = tenderPackList;
		this.remoteProjectBidOpeningListener = remoteProjectBidOpeningListener;
		this.tenderProjectService = new TenderProjectService();
		supplierOpeningResultService = new SupplierOpeningResultService();
	}
	public void start(){
		thread = new Thread(this);
		thread.setName("下载解密信息中..." + tenderProjectEntity.getTenderName());
		thread.start();
	}
	public void stop(){
		thread = null;
		if(null != downloadDecInfoThreadPool){
			downloadDecInfoThreadPool.shutdownNow();
			downloadDecInfoThreadPool = null;
		}
		if(null != downloadBidPriceThreadPoolList && downloadBidPriceThreadPoolList.size() > 0){
			for(ThreadPoolExecutor threadPoolExecutor : downloadBidPriceThreadPoolList){
				if(null != threadPoolExecutor) {
					threadPoolExecutor.shutdownNow();
				}
			}
			downloadBidPriceThreadPoolList.clear();
		}
	}
	public void join(){
		if(null != thread){
			try {
				thread.join();
			} catch (InterruptedException e) {
			}
		}
	}
	
	public void run() {
		
		HttpUrl httpUrl = HttpUtils.getProjectHttpUrl(tenderProjectEntity, transactionServer, PropertiesUtils.getWebserviceProperty(WebserviceConstants.GET_PROJECT_DECRYPT_CONFIRM_STAUTS_METHOD, "bidOpeningHS.do?getProjectDecryptAndConfirmStates"));
		if(null == httpUrl){
			return;
		}
		String openStageType = "00";
		if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
			isPriceStage = true;
			openStageType = "02";
		}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
			isBusAndTecStage = true;
			openStageType = "01";
		}
		httpUrl.setAttribute("openStageType", openStageType);
		StringBuilder stageIds = new StringBuilder();
		Map<String, TenderProjectEntity> tenderPackMap = new HashMap<String, TenderProjectEntity>();
		for(TenderProjectEntity tenderPack : tenderPackList){
			if(stageIds.length() != 0){
				stageIds.append(",");
			}
			stageIds.append(tenderPack.getStageId());
			tenderPackMap.put(tenderPack.getTenderNo(), tenderPack);
		}
		httpUrl.setAttribute("stageIds", stageIds.toString());
		httpUrl.setName("获得远程开标信息");
		httpUrl.setShowSuccessLog(false);
		boolean downloadFlag = resolveXml(HttpUtils.doPost(httpUrl), tenderPackMap);
		if(!downloadFlag) {
			// 不断循环，造 成日志量超大，只有在失败时才记录日志
			ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"获得远程开标信息", httpUrl, downloadFlag);
		}
	}

	
	/**
	 * @Description: 解析返回的xml
	 * @author guoyr
	 * @param xmlStr
	 * @return
	 */
	private boolean resolveXml(HttpResponse httpResponse, Map<String, TenderProjectEntity> tenderPackMap){

		if(null != httpResponse && httpResponse.getStatus()){
			
			if(null != downloadDecInfoThreadPool){
				downloadDecInfoThreadPool.shutdownNow();
				downloadDecInfoThreadPool = null;
			}else {
//				maxThreadPoolCount = 10;
				downloadDecInfoThreadPool = new ThreadPoolExecutor(maxThreadPoolCount, maxThreadPoolCount, 3, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>(),new ThreadPoolExecutor.CallerRunsPolicy());
				//当线程空闲时间达到3秒时，线程会退出，直到线程数量=corePoolSize
				//如果allowCoreThreadTimeout=true，则会直到线程数量=0
				downloadDecInfoThreadPool.allowCoreThreadTimeOut(true);
			}
			
			if(null != downloadBidPriceThreadPoolList && downloadBidPriceThreadPoolList.size() > 0){
				for(ThreadPoolExecutor threadPoolExecutor : downloadBidPriceThreadPoolList){
					if(null != threadPoolExecutor) {
						threadPoolExecutor.shutdownNow();
					}
				}
				downloadBidPriceThreadPoolList.clear();
			}else {
				downloadBidPriceThreadPoolList = new ArrayList<ThreadPoolExecutor>();
			}
			
			JSONArray tenderPackListArray = httpResponse.getDataJSONArray("tenderPackList");
			if(null != tenderPackListArray){
				final boolean needConfirm = PropertiesUtils.getProperty(ConfigConstants.SHOW_CONFIRM_STATUS, "00").equals("01");
				Object[] tenderPackJsonList =  tenderPackListArray.toArray();
				if(null != tenderPackJsonList && tenderPackJsonList.length > 0){
					
					List<Future<List<SupplierOpeningResult>>> tenderPackResultFutureList = new ArrayList<Future<List<SupplierOpeningResult>>>();
					
					for(Object tenderPackObj : tenderPackJsonList){
						final JSONObject tenderPackJson = (JSONObject)tenderPackObj;
						final String packCode = (String)tenderPackJson.get("packCode");
						final TenderProjectEntity tenderPack = tenderPackMap.get(packCode);
						if(null != tenderPack){
							// 是否资审项目
							final boolean isPreEvaluation = (null != tenderPack.getPhaseType() && tenderPack.getPhaseType().equals(TenderProjectEnum.PHASE_TYPE_PRE));
							
							tenderPackResultFutureList.add(downloadDecInfoThreadPool.submit(new Callable<List<SupplierOpeningResult>>() {
								public List<SupplierOpeningResult> call() throws Exception {
									Map<String, Supplier> supplierMap = new HashMap<String, Supplier>();
									JSONArray supplierListArray = (JSONArray)tenderPackJson.get("supplierBidOpeningStatusList");
									if(null != supplierListArray){
										Object[] supplierJsonList =  supplierListArray.toArray(); 
										if(null != supplierJsonList && supplierJsonList.length > 0){
											List<Supplier> supplierList = supplierService.getSupplierOpenBidList(tenderPack.getId());
											if(null != supplierList && supplierList.size() > 0){
												for(Supplier supplier : supplierList){
													SupplierOpeningResult supplierOpeningResult = supplier.getSupplierOpeningResult();
													// 如果废标了，则不需要获取
													if(null != supplierOpeningResult.getRevokeStatus() && supplierOpeningResult.getRevokeStatus().equals(SupplierEnum.REVOKE_YES)){
														continue;
													}
													if(isPriceStage){
														// 如果还未签到，则不获取远程开标信息
//														if(null == supplier.getPriceSignTime()){
//															continue;
//														}
														// 如果已解密完成，且已确认完成不再获取远程开标信息
														if(needConfirm){
															if(null != supplierOpeningResult.getPriceBidStatus() && supplierOpeningResult.getPriceBidStatus().equals(SupplierEnum.OPENBID_SUCCESS) && null != supplierOpeningResult.getPriceConfirmTime()){
																continue;
															}
														}else {
															if(null != supplierOpeningResult.getPriceBidStatus() && supplierOpeningResult.getPriceBidStatus().equals(SupplierEnum.OPENBID_SUCCESS) && null != supplierOpeningResult.getBidPrice()){
																continue;
															}
														}
													}else {
//														if(null == supplier.getSignTime()){
//															continue;
//														}
														// 如果已解密完成，且已确认完成不再获取远程开标信息
														if(needConfirm){
															if(null != supplierOpeningResult.getBusinessBidStatus() && supplierOpeningResult.getBusinessBidStatus().equals(SupplierEnum.OPENBID_SUCCESS) && null != supplierOpeningResult.getConfirmTime()){
																continue;
															}
														}else {
															if(null != supplierOpeningResult.getBusinessBidStatus() && supplierOpeningResult.getBusinessBidStatus().equals(SupplierEnum.OPENBID_SUCCESS)){
																if(isPreEvaluation){
																	// 如果是开标成功，且是资审项目则不在下载开标结果
																	continue;
																}else if(null != supplierOpeningResult.getBidPrice()){
																	// 如果是开标成功，且非资审项目已获取到报价则不在下载开标结果
																	continue;
																}
															}
														}
													}
													// 只获得签到完，并且没有解密完和没有确认完的供应商
													supplierMap.put(supplier.getOrgCode(), supplier);
												}
											}
											if(supplierMap.size() == 0){
												return null;
											}
											
											int maxPoolSize = supplierMap.size() > 5 ? 5 : supplierMap.size();
											ThreadPoolExecutor downloadBidPriceThreadPool = new ThreadPoolExecutor(maxPoolSize, maxPoolSize, 3, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>(),new ThreadPoolExecutor.CallerRunsPolicy());
											//当线程空闲时间达到3秒时，线程会退出，直到线程数量=corePoolSize
											//如果allowCoreThreadTimeout=true，则会直到线程数量=0
											downloadBidPriceThreadPool.allowCoreThreadTimeOut(true);
											downloadBidPriceThreadPoolList.add(downloadBidPriceThreadPool);
											List<Future<SupplierOpeningResult>> tenderPackFutureList = new ArrayList<Future<SupplierOpeningResult>>();
											final BidPriceResolveListener bidPriceResolveListener = new BidPriceResolveListener(tenderPack, maxPoolSize);
											
											for(Object object : supplierJsonList){
												final JSONObject supplierJson = (JSONObject)object;
												String orgCode = (String)supplierJson.get("orgCode");
												final Supplier supplier = supplierMap.get(orgCode);
												if(null != supplier){
													
													String bidOpeningStatus = (String)supplierJson.get("bidOpeningStatus");
													if(null != bidOpeningStatus && bidOpeningStatus.equals("01")){
														bidOpeningStatus = SupplierEnum.OPENBID_SUCCESS;
													}
													if(bidOpeningStatus.isEmpty()){
														continue;
													}
													final boolean dencFlag = null != bidOpeningStatus && bidOpeningStatus.equals(SupplierEnum.OPENBID_SUCCESS);
													
													if(null == thread){
														break;
													}
													
													tenderPackFutureList.add(downloadBidPriceThreadPool.submit(new Callable<SupplierOpeningResult>() {
														public SupplierOpeningResult call() throws Exception {
															SupplierOpeningResult supplierOpeningResult = supplier.getSupplierOpeningResult();
															if(null == supplierOpeningResult.getId()){
																LOG.error(supplier.getSupplierName() + "====开标结果有误，未获取到开标结果=======");
																return null;
															}
															if(!isBusAndTecStage && dencFlag && !isPreEvaluation){
																
																if(null == supplierOpeningResult.getBidPrice()){
																	Future<List<BidPriceResult>> future = bidPriceResolveListener.downloadRemoteBidOpeningResult(supplier);
																	try {
																		List<BidPriceResult> bidPriceResultList = future.get();
																		if(null != bidPriceResultList && null != thread){
																			// 下载投标报价后保存投标报价
																			SupplierOpeningResult _supplierOpeningResult = bidPriceResolveListener.saveSupplierBidPriceResult(supplierOpeningResult, null, null, SupplierEnum.OPENBID_SUCCESS, SupplierEnum.OPENBID_SUCCESS, null, null, bidPriceResultList);
																			if(null == _supplierOpeningResult || null == _supplierOpeningResult.getId()){
																				LOG.error(supplier.getSupplierName() + "[开标结果有误，未获取到开标结果]");
																			}else {
																				supplierOpeningResult = _supplierOpeningResult;
																			}
																		}
																	} catch (InterruptedException e) {
																		LOG.error(supplier.getSupplierName() + "[下载报价线程被中断]");
																	} catch (ExecutionException e) {
																		LOG.error(supplier.getSupplierName() + "[开标结果有误]", e);
																	}
																}
															}
															return resolverSupplierOpeningResult(tenderPack, supplier, supplierOpeningResult, supplierJson, dencFlag);
														}
													}));
												}
											}
											downloadBidPriceThreadPool.shutdown();
											List<SupplierOpeningResult> updateSupplierOpeningResultList = new ArrayList<SupplierOpeningResult>();
											for(Future<SupplierOpeningResult> future : tenderPackFutureList){
												if(null == thread){
													break;
												}
												try {
													SupplierOpeningResult supplierOpeningResult = future.get();
													if(null != supplierOpeningResult){
														updateSupplierOpeningResultList.add(supplierOpeningResult);
													}else {
														LOG.error(tenderPack.getTenderName() + "开标结果有误，未获取到开标结果");
													}
												} catch (InterruptedException e) {
													LOG.error(tenderPack.getTenderName() + "下载开标结果线程被中断");
												} catch (ExecutionException e) {
													LOG.error("", e);
												}
											}
											
											if(null != thread && null != updateSupplierOpeningResultList && updateSupplierOpeningResultList.size() > 0){
												if(isPriceStage){
													supplierOpeningResultService.batchSaveOrUpdate(updateSupplierOpeningResultList, new String[]{"priceBidOpeningTime", "bidOpeningStatus", "priceBidStatus", "priceConfirmTime", "priceConfirmStatus", "revokeStatus"});
												}else if(isBusAndTecStage){
													supplierOpeningResultService.batchSaveOrUpdate(updateSupplierOpeningResultList, new String[]{"bidOpeningTime", "bidOpeningStatus", "businessBidStatus", "technologyBidStatus", "confirmTime", "confirmStatus", "revokeStatus"});
												}else {
													supplierOpeningResultService.batchSaveOrUpdate(updateSupplierOpeningResultList, new String[]{"bidOpeningTime", "bidOpeningStatus", "businessBidStatus", "technologyBidStatus", "priceBidStatus", "confirmTime", "confirmStatus", "revokeStatus"});
												}
												if(null != remoteProjectBidOpeningListener){
													remoteProjectBidOpeningListener.refreshDescInfo(tenderProjectService.getTenderPackOpeningInfo(tenderPack.getId(), bidopeningStage));
												}
											}
											
											return updateSupplierOpeningResultList;
										}
									}
									
									return null;
								} 
							}));
						}
					}
					downloadDecInfoThreadPool.shutdown();
					
					for(Future<List<SupplierOpeningResult>> future : tenderPackResultFutureList){
						if(null == thread){
							break;
						}
						try {
							future.get();
						} catch (InterruptedException e) {
							LOG.error("下载开标结果线程被中断");
						} catch (ExecutionException e) {
//							LOG.error("", e);
						}
					}
				}
				return true;
			}
		}
		return false;
	}
	
	
	
	private SupplierOpeningResult resolverSupplierOpeningResult(TenderProjectEntity tenderPack, Supplier supplier, SupplierOpeningResult supplierOpeningResult, JSONObject supplierJson, boolean dencFlag){
		Date bidOpeningTime = null;
		if(null != supplierJson.get("bidOpeningTime") && !"".equals(supplierJson.get("bidOpeningTime"))){
			bidOpeningTime = DateUtils.toDate((String)supplierJson.get("bidOpeningTime"));//解密状态
			if(null != bidOpeningTime){
				Calendar cal = Calendar.getInstance();
				int nowYear = cal.get(Calendar.YEAR);
				cal.setTime(bidOpeningTime);
				int year = cal.get(Calendar.YEAR);
				// 发现从接口中传过来的时间少于1900
				if(year < 1900){
					cal.set(Calendar.YEAR, nowYear);
				}
				bidOpeningTime = cal.getTime();
			}else {
				LOG.error("时间格式化错误："+ supplierJson.toString());
			}
		}
		
		Date confirmTime = null;
		if(null != supplierJson.get("confirmTime") && !"".equals(supplierJson.get("confirmTime"))){
			confirmTime = DateUtils.toDate((String)supplierJson.get("confirmTime"));
			if(null != confirmTime){
				Calendar cal = Calendar.getInstance();
				int nowYear = cal.get(Calendar.YEAR);
				cal.setTime(confirmTime);
				int year = cal.get(Calendar.YEAR);
				// 发现从接口中传过来的时间少于1900
				if(year < 1900){
					cal.set(Calendar.YEAR, nowYear);
				}
				confirmTime = cal.getTime();
			}else {
				LOG.error("时间格式化错误："+ supplierJson.toString());
			}
		}
		String confirmStatus = (String)supplierJson.get("confirmStatus");
		
		String supplierConfirmStatus = null;
		if(null != confirmStatus && confirmStatus.length() > 0){
			if(confirmStatus.equals("01")){
				supplierConfirmStatus = SupplierEnum.CONFIRM_YES;
			}else if(confirmStatus.equals("02")){
				supplierConfirmStatus = SupplierEnum.CONFIRM_QUESTION;
			}else if(confirmStatus.equals("03")){
				supplierConfirmStatus = SupplierEnum.CONFIRM_TIME_OUT;
			}
		}
		supplierOpeningResult.setTenderId(tenderPack.getId());
		supplierOpeningResult.setSupplierId(supplier.getId());
		String bidOpeningStatus = (String)supplierJson.get("bidOpeningStatus");
		if(dencFlag){
			bidOpeningStatus = SupplierEnum.OPENBID_SUCCESS;
			if(null == supplierOpeningResult.getBidOpeningStatus() || !supplierOpeningResult.getBidOpeningStatus().equals(SupplierEnum.OPENBID_SUCCESS)){
				supplierOpeningResult.setBidOpeningStatus(bidOpeningStatus);
			}
		}
		supplierOpeningResult.setRevokeStatus(SupplierEnum.REVOKE_NO);
		if(isPriceStage){
			supplierOpeningResult.setPriceBidOpeningTime(bidOpeningTime);
			supplierOpeningResult.setPriceBidStatus(bidOpeningStatus);
			supplierOpeningResult.setPriceConfirmTime(confirmTime);
			supplierOpeningResult.setPriceConfirmStatus(supplierConfirmStatus);
		}else if(isBusAndTecStage){
			supplierOpeningResult.setBidOpeningTime(bidOpeningTime);
			supplierOpeningResult.setBusinessBidStatus(bidOpeningStatus);
			supplierOpeningResult.setTechnologyBidStatus(bidOpeningStatus);
			supplierOpeningResult.setConfirmTime(confirmTime);
			supplierOpeningResult.setConfirmStatus(supplierConfirmStatus);
		}else {
			supplierOpeningResult.setBidOpeningTime(bidOpeningTime);
			supplierOpeningResult.setBusinessBidStatus(bidOpeningStatus);
			supplierOpeningResult.setTechnologyBidStatus(bidOpeningStatus);
			supplierOpeningResult.setPriceBidStatus(bidOpeningStatus);
			supplierOpeningResult.setConfirmStatus(supplierConfirmStatus);
			supplierOpeningResult.setConfirmTime(confirmTime);
		}
		return supplierOpeningResult;
	}
}
