package com.gx.obe.opening.thread;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import com.gx.obe.WebserviceConstants;
import com.gx.obe.business.http.HttpResponse;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.opening.listener.BatchRemoteSignListener;
import com.gx.obe.util.utils.DateUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.TransactionServer;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.SupplierService;
import com.gx.obe.web.service.TenderProjectService;
import com.gx.utils.ActionLogUtils;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

/**
 * @Description: 获得项目下的包件的签到信息
 * @author guoyr
 */
public class RemoteProjectSignInfoThread implements Runnable{
	
	private Logger LOG = Logger.getLogger(RemoteProjectSignInfoThread.class);
	private TenderProjectEntity tenderProjectEntity;
	private TransactionServer transactionServer;
	private List<TenderProjectEntity> tenderPackList;
	private String bidopeningStage;
	private Thread thread;
	private SupplierService supplierService = new SupplierService();
	private TenderProjectService tenderProjectService ;
	private boolean isPriceStage = false;
	private List<String> signComplateSupplierIdList;
	private BatchRemoteSignListener remoteProjectSignListener;
	
	public RemoteProjectSignInfoThread(TenderProjectEntity tenderProjectEntity, String bidopeningStage, TransactionServer transactionServer, List<TenderProjectEntity> tenderPackList, BatchRemoteSignListener remoteProjectSignListener){
		this.tenderProjectEntity = tenderProjectEntity;
		this.bidopeningStage = bidopeningStage;
		this.transactionServer = transactionServer;
		this.tenderPackList = tenderPackList;
		this.remoteProjectSignListener = remoteProjectSignListener;
		tenderProjectService = new TenderProjectService();
	}
	public void start(){
		thread = new Thread(this);
		thread.setName("下载签到信息中..." + tenderProjectEntity.getTenderName());
		thread.start();
	}
	public void stop(){
		thread = null;
		if(null != signComplateSupplierIdList){
			signComplateSupplierIdList.clear();
		}
	}
	public void join(){
		if(null != thread){
			try {
				thread.join();
			} catch (InterruptedException e) {
			}
		}
	}
	
	public void run() {
		
		HttpUrl httpUrl = HttpUtils.getProjectHttpUrl(tenderProjectEntity, transactionServer, PropertiesUtils.getWebserviceProperty(WebserviceConstants.GET_PROJECT_SIGN_MESSAGE_METHOD, "bidOpeningHS.do?getProjectBidOpeningSignInfoList"));
		httpUrl.setName("获得远程签到信息");
		String openStageType = "00";
		if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
			isPriceStage = true;
			openStageType = "02";
		}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
			openStageType = "01";
		}
		httpUrl.setAttribute("openStageType", openStageType);
		StringBuilder stageIds = new StringBuilder();
		Map<String, TenderProjectEntity> tenderPackMap = new HashMap<String, TenderProjectEntity>();
		for(TenderProjectEntity tenderPack : tenderPackList){
			if(stageIds.length() != 0){
				stageIds.append(",");
			}
			stageIds.append(tenderPack.getStageId());
			tenderPackMap.put(tenderPack.getTenderNo(), tenderPack);
		}
		httpUrl.setAttribute("stageIds", stageIds.toString());
		
		boolean flag = resolveXml(HttpUtils.doPost(httpUrl), tenderPackMap);
		if(!flag) {
			// 不断循环，造 成日志量超大，只有在失败时才记录日志
			ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"获得远程签到信息", httpUrl, flag);
		}
	}

	
	/**
	 * @Description: 解析返回的xml
	 * @author guoyr
	 * @param xmlStr
	 * @return
	 */
	private boolean resolveXml(HttpResponse httpResponse, Map<String, TenderProjectEntity> tenderPackMap){

		if(null != httpResponse && httpResponse.getStatus()){
			
			JSONArray tenderPackListArray = httpResponse.getDataJSONArray("tenderPackList");
			if(null != tenderPackListArray){
				Object[] tenderPackJsonList =  tenderPackListArray.toArray();
				if(null != tenderPackJsonList && tenderPackJsonList.length > 0){
					for(Object tenderPackObj : tenderPackJsonList){
						if(null == thread){
							break;
						}
						JSONObject tenderPackJson = (JSONObject)tenderPackObj;
						String packCode = (String)tenderPackJson.get("packCode");
						TenderProjectEntity tenderPack = tenderPackMap.get(packCode);
						if(null != tenderPack){
							
							JSONArray supplierListArray = (JSONArray)tenderPackJson.get("supplierSignInfoList");
							List<Supplier> updateSupplierList = new ArrayList<Supplier>();
							if(null != supplierListArray){
								Object[] supplierJsonList =  supplierListArray.toArray();
								if(null != supplierJsonList && supplierJsonList.length > 0){
									Map<String, Supplier> supplierMap = new HashMap<String, Supplier>();
									List<Supplier> supplierList = supplierService.getAllSupplierList(tenderPack.getId());
									if(null != supplierList && supplierList.size() > 0){
										for(Supplier supplier : supplierList){
											if(isPriceStage){
												if(null != supplier.getPriceSignTime()){
													continue;
												}
											}else {
												if(null != supplier.getSignTime()){
													continue;
												}
											}
											supplierMap.put(supplier.getOrgCode(), supplier);
										}
									}
										
									for(Object object : supplierJsonList){
										if(null == thread){
											break;
										}
										JSONObject supplierJson = (JSONObject)object;
										String orgCode = (String)supplierJson.get("orgCode");
										Supplier supplier = supplierMap.get(orgCode);
										if(null != supplier){
											
											Date signTime = null;
											if(null != supplierJson.get("signTime") && !"".equals(supplierJson.get("signTime"))){
												signTime = DateUtils.toDate((String)supplierJson.get("signTime"));
												if(null != signTime){
													Calendar cal = Calendar.getInstance();
													int nowYear = cal.get(Calendar.YEAR);
													cal.setTime(signTime);
													int year = cal.get(Calendar.YEAR);
													// 发现从接口中传过来的时间少于1900
													if(year < 1900){
														cal.set(Calendar.YEAR, nowYear);
													}
													signTime = cal.getTime();
												}else {
													LOG.error("时间格式化错误："+ httpResponse.getContent());
												}
											}
											String signPerson = (String)supplierJson.get("signPerson");//签到人
											String signPersonTel = (String)supplierJson.get("signPersonTel");//签到人电话
											String signPersonCard = (String)supplierJson.get("signPersonCard");
											if(null == signTime || null == signPerson || signPerson.length() == 0){
												continue;
											}
											String email = (String)supplierJson.get("email");
											if(null != email && email.length() > 0){
												supplier.setEmail(email);
											}
											if(isPriceStage){
												supplier.setPriceSignPerson(signPerson);
												supplier.setPriceSignTime(signTime);
												supplier.setPriceSignPersonTel(signPersonTel);
												supplier.setPriceSignPersonCard(signPersonCard);
											}else {
												supplier.setSignPerson(signPerson);
												supplier.setSignTime(signTime);
												supplier.setSignPersonTel(signPersonTel);
												supplier.setSignPersonCard(signPersonCard);
											}
											updateSupplierList.add(supplier);
										}
									
									}
								}
							}
							if(null != thread && null != updateSupplierList && updateSupplierList.size() > 0){
								if(isPriceStage){
									supplierService.batchUpdateProperty(updateSupplierList, new String[]{"priceSignTime", "priceSignPerson", "priceSignPersonTel", "priceSignPersonCard"});
								}else {
									supplierService.batchUpdateProperty(updateSupplierList, new String[]{"signTime", "signPerson", "signPersonTel", "signPersonCard","email"});
								}
								remoteProjectSignListener.refreshSignInfo(tenderProjectService.getTenderPackOpeningInfo(tenderPack.getId(), bidopeningStage));
								return true;
							}
						}
					}
					
				}
			}
		}
		return false;
	}
	
}
