package com.gx.obe.opening.thread;

import java.io.File;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

import com.gx.obe.WebserviceConstants;
import com.gx.obe.business.enumeration.SupplierEnum;
import com.gx.obe.business.http.HttpResponse;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.thread.DownloadRemoteBidOpeningQuestionThread;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.SupplierOpeningResult;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.SupplierOpeningResultService;
import com.gx.obe.web.service.SupplierService;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

/**
 * @Description: 获得供应商远程签到信息
 * @author guoyr
 */
public class RemoteSupplierConfirmThread implements Runnable{
	
	private Logger LOG = Logger.getLogger(RemoteSupplierConfirmThread.class);
	private TenderProjectEntity tenderProjectEntity;
	private String bidopeningStage;
	private Thread thread;
	private RemoteConfirmListener checkedConfirmListener;
	private SupplierOpeningResultService supplierOpeningResultService;
	private SupplierService supplierService = new SupplierService();
	private Map<String, SupplierOpeningResult> confirmInfoMap;
	private boolean isPriceStage = false;;
	private List<String> signComplateSupplierIdList;
	
	public RemoteSupplierConfirmThread(TenderProjectEntity tenderProjectEntity, String bidopeningStage, RemoteConfirmListener checkedConfirmListener){
		this.tenderProjectEntity = tenderProjectEntity;
		this.bidopeningStage = bidopeningStage;
		this.checkedConfirmListener = checkedConfirmListener;
		supplierOpeningResultService = new SupplierOpeningResultService();
	}
	public void start(){
		thread = new Thread(this);
		thread.start();
	}
	public void stop(){
		thread = null;
		if(null != signComplateSupplierIdList){
			signComplateSupplierIdList.clear();
		}
		if(null != confirmInfoMap){
			confirmInfoMap.clear();
		}
	}
	public void join(){
		if(null != thread){
			try {
				thread.join();
			} catch (InterruptedException e) {
			}
		}
	}
	
	public synchronized void refreshConfirmStatus(Supplier supplier, Date confirmTime, String confirmStatus){
		if(!signComplateSupplierIdList.contains(supplier.getId())){
			signComplateSupplierIdList.add(supplier.getId());
			checkedConfirmListener.refreshConfirmInfo(supplier, confirmTime, confirmStatus);
		}
	}
	public void run() {
		
		HttpUrl httpUrl = HttpUtils.getProjectHttpUrl(tenderProjectEntity, PropertiesUtils.getWebserviceProperty(WebserviceConstants.GET_SUPPLIER_DECRYPT_STAUTS_METHOD, ""));
		if(null != httpUrl){
			httpUrl.setName("获取供应商远程确认结果");
			// openStageType : [00一次开标阶段，01非价格开标阶段，02价格开标阶段]
			String openStageType = "00";
			if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
				isPriceStage = true;
				openStageType = "02";
			}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
				openStageType = "01";
			}
			httpUrl.setAttribute("openStageType", openStageType);
			signComplateSupplierIdList = new ArrayList<String>();
			confirmInfoMap = new HashMap<String, SupplierOpeningResult>();
			// 标记是否下载过质疑文件
			Map<String, Boolean> downQuestionMap = new HashMap<String, Boolean>();
			Set<String> supplierOrgCodeSet = new HashSet<String>(); //本地未签到供应商机构编码
			while(null != thread){
				LOG.debug("获取供应商远程确认结果中...");
				try {
					Thread.sleep(1500);
				} catch (InterruptedException e) {
				}
				List<Supplier> supplierList = supplierService.getSupplierOpenBidList(tenderProjectEntity.getId());
				if(null != supplierList && supplierList.size() > 0){
					supplierOrgCodeSet.clear();
					for(Supplier supplier : supplierList){
						SupplierOpeningResult supplierOpeningResult = supplier.getSupplierOpeningResult();
						if(null != supplierOpeningResult && null != supplierOpeningResult.getId()){
							// 如果废标了，则不需要获取
							if(null != supplierOpeningResult.getRevokeStatus() && supplierOpeningResult.getRevokeStatus().equals(SupplierEnum.REVOKE_YES)){
								continue;
							}
							
							Date confirmTime = supplierOpeningResult.getConfirmTime();
							String confirmStatus = supplierOpeningResult.getConfirmStatus();
							if(isPriceStage){
								confirmTime = supplierOpeningResult.getPriceConfirmTime();
								confirmStatus = supplierOpeningResult.getPriceConfirmStatus();
							}
							if(null != confirmStatus && confirmStatus.length() > 0){
								// 如果当前供应商提出质疑，则下载质疑文件
								if(confirmStatus.equals(SupplierEnum.CONFIRM_QUESTION) && null == downQuestionMap.get(supplier.getOrgCode())){
									File supplierQuestinFile = new File(FileConstants.getBidFileQuestionPath(supplier.getTenderId(), supplier.getId()) + supplier.getId() + "_"+bidopeningStage+".pdf");
									if(null == supplierQuestinFile || !supplierQuestinFile.exists()){
										DownloadRemoteBidOpeningQuestionThread downloadBidOpeningQuestionThread = new DownloadRemoteBidOpeningQuestionThread(tenderProjectEntity, supplier, bidopeningStage, null);
										downloadBidOpeningQuestionThread.start();
									}
									// 标记已下载过质疑文件
									downQuestionMap.put(supplier.getOrgCode(), true);
								}
								refreshConfirmStatus(supplier, confirmTime, confirmStatus);
								continue;
							}
							confirmInfoMap.put(supplier.getOrgCode(), supplierOpeningResult);
						}
						supplierOrgCodeSet.add(supplier.getOrgCode());
					}
					// 如果存在需要查询确认状态的供应商则调用接口
					if(supplierOrgCodeSet.size() > 0){
						resolveXml(HttpUtils.doPost(httpUrl), supplierOrgCodeSet);
					}else {
						// 如果不存在需要刷新确认状态的供应商，则结束当前线程，否则继续查询
						thread = null;
						break;
						
					}
				}
			}
			LOG.debug("获取供应商远程确认结果结束！");
			downQuestionMap.clear();
			downQuestionMap = null;
		}
	}

//	
//	/**
//	 * @Description: 下载供应商签到信息
//	 * @author guoyr
//	 * @param httpUrl
//	 * @param supplierOrgNoSet
//	 * @return 
//	 */
//	public List<SupplierOpeningResult> getRemoteSignList(HttpUrl httpUrl, Set<String> supplierOrgNoSet, Map<String, SupplierOpeningResult> supplierOpeningResultMap){
//		try {
//			String getWebserviceXml = HttpUtils.post(httpUrl);
//			return resolveXml(getWebserviceXml, supplierOrgNoSet, supplierOpeningResultMap);
//		} catch (Exception e) {
//		}
//		return null;
//	}
	
	
	/**
	 * @Description: 解析返回的xml
	 * @author guoyr
	 * @param xmlStr
	 * @return
	 */
	private List<SupplierOpeningResult> resolveXml(HttpResponse httpResponse, Set<String> supplierOrgCodeSet){
		List<SupplierOpeningResult> updateConfirmInfoList = null;
		try {
			if(null != httpResponse && httpResponse.getStatus()){
				SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
				JSONArray supplierListArray = httpResponse.getDataJSONArray("supplierBidOpeningStatusList");
				if(null != supplierListArray){
					Object[] supplierJsonList =  supplierListArray.toArray();
					if(null != supplierJsonList && supplierJsonList.length > 0){
						updateConfirmInfoList = new ArrayList<SupplierOpeningResult>();
						
						for(Object object : supplierJsonList){
							JSONObject supplierJson = (JSONObject)object;
							String orgCode = (String)supplierJson.get("orgCode");
							// 过滤只有未签到的供应商进入要更新列表
							if(supplierOrgCodeSet.contains(orgCode)){
								Date confirmTime = null;
								try {
									if(null != supplierJson.get("confirmTime") && !"".equals(supplierJson.get("confirmTime"))){
										confirmTime = format.parse((String)supplierJson.get("confirmTime"));
									}
								} catch (ParseException e) {
									LOG.error("时间格式化错误："+ e.getMessage());
								}
								String confirmStatus = (String)supplierJson.get("confirmStatus");
								if(null != confirmStatus && confirmStatus.length() > 0){
									SupplierOpeningResult supplierOpeningResult = confirmInfoMap.get(orgCode);
									if(null != supplierOpeningResult){
										String supplierConfirmStatus = null;
										if(confirmStatus.equals("01")){
											supplierConfirmStatus = SupplierEnum.CONFIRM_YES;
										}else if(confirmStatus.equals("02")){
											supplierConfirmStatus = SupplierEnum.CONFIRM_QUESTION;
										}else if(confirmStatus.equals("03")){
											supplierConfirmStatus = SupplierEnum.CONFIRM_TIME_OUT;
										}
										if(isPriceStage){
											supplierOpeningResult.setPriceConfirmStatus(supplierConfirmStatus);
											supplierOpeningResult.setPriceConfirmTime(confirmTime);
										}else {
											supplierOpeningResult.setConfirmStatus(supplierConfirmStatus);
											supplierOpeningResult.setConfirmTime(confirmTime);
										}
										updateConfirmInfoList.add(supplierOpeningResult);
									}
								}
							}
						}
					}
				}
			
			}
		} catch (Exception e) {
			LOG.error("返回的xml结果为："+httpResponse.getContent(), e);
		}finally{
			if(null != thread && null != updateConfirmInfoList && updateConfirmInfoList.size() > 0){
				if(isPriceStage){
					supplierOpeningResultService.batchUpdateProperty(updateConfirmInfoList, new String[]{"priceConfirmStatus", "priceConfirmTime"});
				}else {
					supplierOpeningResultService.batchUpdateProperty(updateConfirmInfoList, new String[]{"confirmStatus", "confirmTime"});
				}
			}
		}
		return updateConfirmInfoList;
	}
	
	public static interface RemoteConfirmListener{
		public void refreshConfirmInfo(Supplier supplier, Date confirmDate, String confirmStatus);
	}
	
}
