package com.gx.obe.opening.thread;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

import com.gx.obe.WebserviceConstants;
import com.gx.obe.business.http.HttpResponse;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.common.widget.thread.MyRunnable;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.SupplierService;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

/**
 * @Description: 获得供应商远程签到信息
 * @author guoyr
 */
public class RemoteSupplierSignThread extends MyRunnable{
	
	private Logger LOG = Logger.getLogger(RemoteSupplierSignThread.class);
	private TenderProjectEntity tenderProjectEntity;
	private String bidopeningStage;
	private Thread thread;
	private RemoteSignListener checkedSignListener;
	private SupplierService supplierService = new SupplierService();
	private Map<String, Supplier> signInfoMap;
	private boolean isPriceStage = false;
	private List<String> signComplateSupplierIdList;
	
	public RemoteSupplierSignThread(TenderProjectEntity tenderProjectEntity, String bidopeningStage, RemoteSignListener checkedSignListener){
		this.tenderProjectEntity = tenderProjectEntity;
		this.bidopeningStage = bidopeningStage;
		this.checkedSignListener = checkedSignListener;
	}
	public void start(){
		thread = new Thread(this);
		thread.start();
	}
	public void stop(){
		thread = null;
		if(null != signComplateSupplierIdList){
			signComplateSupplierIdList.clear();
		}
		if(null != signInfoMap){
			signInfoMap.clear();
		}
	}
	public void join(){
		if(null != thread){
			try {
				thread.join();
			} catch (InterruptedException e) {
			}
		}
	}
	
	public synchronized void refreshSignStatus(Supplier supplier, Date signTime, String signPerson, String signPersonTel, String signPersonCard){
		if(!signComplateSupplierIdList.contains(supplier.getId())){
			signComplateSupplierIdList.add(supplier.getId());
			checkedSignListener.refreshSignInfo(supplier, signTime, signPerson, signPersonTel, signPersonCard);
		}
	}
	
	public void run() {
		
		HttpUrl httpUrl = HttpUtils.getProjectHttpUrl(tenderProjectEntity, PropertiesUtils.getWebserviceProperty(WebserviceConstants.GET_SUPPLIER_SIGN_MESSAGE_METHOD, ""));
		if(null != httpUrl){
			httpUrl.setName("获取供应商远程签到结果");
			String openStageType = "00";
			if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
				isPriceStage = true;
				openStageType = "02";
			}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
				openStageType = "01";
			}
			httpUrl.setAttribute("openStageType", openStageType);
		}
		signComplateSupplierIdList = new ArrayList<String>();
		signInfoMap = new HashMap<String, Supplier>();
		Set<String> supplierOrgCodeSet = new HashSet<String>(); //本地未签到供应商机构编码
		long startTime = 0;
		long useTime = -1;
		while(null != thread){
			startTime = Calendar.getInstance().getTimeInMillis();
			LOG.debug("获取供应商远程签到结果中...");
			List<Supplier> supplierList = supplierService.getSupplierOpenBidList(tenderProjectEntity.getId());
			if(null != supplierList && supplierList.size() > 0){
				supplierOrgCodeSet.clear();
				int signComplateCount = 0;
				for(Supplier supplier : supplierList){
//					SupplierOpeningResult supplierOpeningResult = supplier.getSupplierOpeningResult();
					// 判断投标人是否废标（如果跳过的话，当投标人废标后，就无法显示签到信息了）
//					if(null != supplierOpeningResult && null != supplierOpeningResult.getRevokeStatus() && supplierOpeningResult.getRevokeStatus().equals(SupplierEnum.REVOKE_YES)){
//						continue;
//					}
					boolean signComplate = false;
					if(isPriceStage){
						if(null != supplier.getPriceSignTime()){
							refreshSignStatus(supplier, supplier.getPriceSignTime(), supplier.getPriceSignPerson(), supplier.getPriceSignPersonTel(), supplier.getPriceSignPersonCard());
							signComplate = true;
						}
					}else {
						if(null != supplier.getSignTime()){
							refreshSignStatus(supplier, supplier.getSignTime(), supplier.getSignPerson(), supplier.getSignPersonTel(), supplier.getSignPersonCard());
							signComplate = true;
						}
					}
					if(signComplate){
						signComplateCount ++;
					}else {
						
						supplierOrgCodeSet.add(supplier.getOrgCode());
						signInfoMap.put(supplier.getOrgCode(), supplier);
					}
				}
				checkedSignListener.refreshSignCount(signComplateCount);
				// 如果存在需要查询签到状态的供应商则调用接口
				if(supplierOrgCodeSet.size() > 0 && null != httpUrl){
					// 每隔3秒刷新
					if(useTime > 5000){
					}else if(useTime > 0) {
						try {
							Thread.sleep(5000 - useTime);
						} catch (InterruptedException e) {
						}
						
					}
					resolveXml(HttpUtils.doPost(httpUrl), supplierOrgCodeSet);
				}else {
					thread = null;
					break;
				}
			}
			useTime = Calendar.getInstance().getTimeInMillis() - startTime;
		}
		LOG.debug("获取供应商远程签到结果结束！");
	
	}

	
	/**
	 * @Description: 解析返回的xml
	 * @author guoyr
	 * @param xmlStr
	 * @return
	 */
	private void resolveXml(HttpResponse httpResponse, Set<String> supplierOrgCodeSet){
		List<Supplier> updateSupplierList = new ArrayList<Supplier>();
		try {
			if(null != httpResponse && httpResponse.getStatus()){
				SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
				JSONArray supplierListArray = httpResponse.getDataJSONArray("bidOpeningSignInfoList");
				if(null != supplierListArray){
					Object[] supplierJsonList =  supplierListArray.toArray();
					if(null != supplierJsonList && supplierJsonList.length > 0){
						for(Object object : supplierJsonList){
							JSONObject supplierJson = (JSONObject)object;
							String orgCode = (String)supplierJson.get("orgCode");
							// 过滤只有未签到的供应商进入要更新列表
							if(supplierOrgCodeSet.contains(orgCode)){
								Supplier supplier = signInfoMap.get(orgCode);
								if(null != supplier){
									Date signTime = null;
									try {
										if(null != supplierJson.get("signTime") && !"".equals(supplierJson.get("signTime"))){
											signTime = format.parse((String)supplierJson.get("signTime"));
										}
									} catch (ParseException e) {
										LOG.error("时间格式化错误："+ e.getMessage());
									}
									String signPerson = (String)supplierJson.get("signPerson");//签到人
									String signPersonTel = (String)supplierJson.get("signPersonTel");//签到人电话
									String signPersonCard = (String)supplierJson.get("signPersonCard");
									if(null == signTime || null == signPerson || signPerson.length() == 0){
										continue;
									}
									String email = (String)supplierJson.get("email");
									if(null != email && email.length() > 0){
										supplier.setEmail(email);
									}
									if(isPriceStage){
										supplier.setPriceSignPerson(signPerson);
										supplier.setPriceSignTime(signTime);
										supplier.setPriceSignPersonTel(signPersonTel);
										supplier.setPriceSignPersonCard(signPersonCard);
									}else {
										supplier.setSignPerson(signPerson);
										supplier.setSignTime(signTime);
										supplier.setSignPersonTel(signPersonTel);
										supplier.setSignPersonCard(signPersonCard);
									}
									updateSupplierList.add(supplier);
								}
							}
						}
					}
				}
			
			}
		} catch (Exception e) {
			LOG.error("获取远程签到信息失败，返回的xml结果为："+httpResponse.getContent(), e);
		}finally{
			if(null != thread && null != updateSupplierList && updateSupplierList.size() > 0){
				if(isPriceStage){
					supplierService.batchUpdateProperty(updateSupplierList, new String[]{"priceSignTime", "priceSignPerson", "priceSignPersonTel", "priceSignPersonCard"});
				}else {
					supplierService.batchUpdateProperty(updateSupplierList, new String[]{"signTime", "signPerson", "signPersonTel", "signPersonCard"});
				}
			}
		}
	}
	
	public static interface RemoteSignListener{
		public void refreshSignInfo(Supplier supplier, Date signTime, String signPerson, String signPersonTel, String signPersonCard);
		public void refreshSignCount(int signCount);
	}
	
}
