package com.gx.obe.opening.thread;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import com.gx.obe.WebserviceConstants;
import com.gx.obe.business.http.HttpResponse;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.web.entity.tender.TenderProjectEntity;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

/**
 * @Description: 获得供应商远程签到信息
 * @author guoyr
 */
public class RomoteSupplierOnlineThread implements Runnable{
	
	private Logger LOG = Logger.getLogger(RomoteSupplierOnlineThread.class);
	private TenderProjectEntity tenderProjectEntity;
	private Thread thread;
	private EnterOpenListener checkOnlineListener;
	private Map<String, String> statusMap;
	private String bidopeningStage;
	
	public RomoteSupplierOnlineThread(TenderProjectEntity tenderProjectEntity, String bidopeningStage, EnterOpenListener checkOnlineListener){
		this.tenderProjectEntity = tenderProjectEntity;
		this.bidopeningStage = bidopeningStage;
		this.checkOnlineListener = checkOnlineListener;
	}
	public void start(){
		thread = new Thread(this);
		thread.setName(tenderProjectEntity.getTenderName());
		thread.start();
	}
	public void stop(){
		thread = null;
	}
	public void join(){
		if(null != thread){
			try {
				thread.join();
			} catch (InterruptedException e) {
			}
		}
	}
	public void run() {
		
		HttpUrl httpUrl = HttpUtils.getProjectHttpUrl(tenderProjectEntity, PropertiesUtils.getWebserviceProperty(WebserviceConstants.GET_SUPPLIER_SIGN_MESSAGE_METHOD, ""));
		if(null != httpUrl){
			httpUrl.setName("获取供应商远程在线状态");
			// openStageType : [00一次开标阶段，01非价格开标阶段，02价格开标阶段]
			String openStageType = "00";
			if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
				openStageType = "02";
			}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
				openStageType = "01";
			}
			httpUrl.setAttribute("openStageType", openStageType);
			statusMap = new HashMap<String, String>();
			while(null != thread){
				LOG.debug("获取供应商远程在线状态中...");
				resolveXml(HttpUtils.doPost(httpUrl));
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
				}
			}
			LOG.debug("获取供应商远程在线状态结束！");
		}
	}
	/**
	 * @Description: 解析返回的xml
	 * @author guoyr
	 * @param xmlStr
	 * @return
	 */
	private void resolveXml(HttpResponse httpResponse){
		
		if(null != httpResponse && httpResponse.getStatus()){
			try {
				statusMap.clear();
				
				JSONArray supplierListArray = httpResponse.getDataJSONArray("bidOpeningSignInfoList");
				Object[] supplierJsonList =  supplierListArray.toArray();
				if(null != supplierJsonList && supplierJsonList.length > 0){
					for(Object object : supplierJsonList){
						JSONObject supplierJson = (JSONObject)object;
						String orgCode = (String)supplierJson.get("orgCode");
						String status = ((String)supplierJson.get("enterOpenRoomStatus"));//签到人
						// 00:未进入; 01:已进入; 02:已退出
						if(null != orgCode && null != status){
							statusMap.put(orgCode, status);
						}
					}
				}
				
			} catch (Exception e) {
				LOG.error("返回的xml结果为："+httpResponse.getContent(), e);
			}finally{
				// 如果下载到签到结果，则刷新
				if(null != thread && null != checkOnlineListener){
					checkOnlineListener.enterStatus(statusMap);
				}
			}
		}
	}
	
	public static interface EnterOpenListener{
		public void enterStatus(Map<String, String> enterStatusMap);
	}
	
}
