package com.gx.obe.opening.thread;

import java.text.MessageFormat;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;

import com.gx.obe.WebserviceConstants;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.common.file.thread.CheckHttpBidFileThread;
import com.gx.obe.common.widget.listener.ProgressMessageListener;
import com.gx.obe.common.widget.thread.MyRunnable;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.DateUtils;
import com.gx.obe.web.entity.price.BidPrice;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.BidPriceService;
import com.gx.obe.web.service.SupplierService;
import com.gx.obe.web.service.TenderProjectService;
import com.gx.tools.utils.BidPriceUtils;
import com.gx.utils.ActionLogUtils;
import com.gx.utils.Params;

/** 
 * @Description: 启动远程解密
 * @author guoyr 
 */
public class StartRemoteDecryptBidFilThread extends MyRunnable{
	
	private TenderProjectEntity tenderProjectEntity;
	private Date decryptEndTime;
	private final String TIMESTAMP_STR = "yyyy-MM-dd HH:mm:ss";
	private Thread thread;
	private ProgressMessageListener listener;
	private String bidopeningStage;
	private Logger LOG = Logger.getLogger(CheckHttpBidFileThread.class);
	private boolean isPriceStage = false;
	private boolean isBusAndTecStage = false;
	private boolean synchronizedInfoOnly = false;
	private BidPriceService bidPriceService;
	private TenderProjectService tenderProjectService;
	private SupplierService supplierService = new SupplierService();

	public StartRemoteDecryptBidFilThread(TenderProjectEntity tenderProjectEntity, Date decryptEndTime, String bidopeningStage, ProgressMessageListener listener, boolean synchronizedInfoOnly){
		this.tenderProjectEntity = tenderProjectEntity;
		this.decryptEndTime = decryptEndTime;
		this.listener = listener;
		this.bidopeningStage =bidopeningStage;
		this.synchronizedInfoOnly =synchronizedInfoOnly;
		tenderProjectService = new TenderProjectService();
		bidPriceService = new BidPriceService();
	}
	public void start(){
		thread = new Thread(this);
		thread.start();
	}
	public void stop(){
		thread = null;
	}
	public void join(){
		if(null != thread){
			try {
				thread.join();
			} catch (InterruptedException e) {
			}
		}
	}
	public void run(){

		// 先加载报价结构
		loadBidPrice();
		
		boolean startRemoteDecrypt = false;
		String openStageType = "00";
		if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
			isPriceStage = true;
			openStageType = "02";
		}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
			isBusAndTecStage = true;
			openStageType = "01";
		}
	
		if(synchronizedInfoOnly){
			startRemoteDecrypt = true;
			ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"仅同步远程解密", new Params("tenderNo", tenderProjectEntity.getTenderNo()).add("openStageType", openStageType));
		}else {
			listener.message(Messages.Tips_starting, 0);
			HttpUrl httpUrl = HttpUtils.getProjectHttpUrl(tenderProjectEntity, PropertiesUtils.getWebserviceProperty(WebserviceConstants.START_REMOTE_DECRYPT_METHOD, ""));
			if(null != httpUrl){
				httpUrl.setName("启动远程解密");
				httpUrl.setAttribute("openStageType", openStageType);
				httpUrl.setAttribute("bidOpeningDeadLine", null != decryptEndTime ? DateUtils.getFormat(decryptEndTime, TIMESTAMP_STR) : null);// 解密截止时间
				
				startRemoteDecrypt = HttpUtils.doPost(httpUrl).getStatus();
				ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"启动远程解密", httpUrl, startRemoteDecrypt);
				if(!startRemoteDecrypt) {
					startRemoteDecrypt = HttpUtils.doPost(httpUrl).getStatus();
					ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"第二次启动远程解密", httpUrl, startRemoteDecrypt);
				}
			}
		}

		if(startRemoteDecrypt){
			// 初始化供应商开标结果
			supplierService.initSupplierBidOpeningResult(tenderProjectEntity.getId());
			if(isPriceStage){
				tenderProjectEntity.setStartPriceBidOpening(CommonEnum.YES);
				tenderProjectService.updateAssignProperty(tenderProjectEntity, new String[]{"id","startPriceBidOpening"});
			}else {
				tenderProjectEntity.setStartBidOpening(CommonEnum.YES);
				tenderProjectService.updateAssignProperty(tenderProjectEntity, new String[]{"id","startBidOpening"});
			}
			
			tenderProjectService.updateBidOpeningStatus(tenderProjectEntity, bidopeningStage, CommonEnum.IN_PROGRESS);
			if(synchronizedInfoOnly) {
				listener.message(Messages.Tips_synchronizDecryptInfoFail, -1);
			}else {
				listener.message(MessageFormat.format(Messages.Tips_startSuccess, Messages.Button_decryptRemote), -1);
			}
		}else {
			if(synchronizedInfoOnly) {
				listener.message(Messages.Tips_synchronizDecryptInfoFail, -1);
			}else {
				listener.message(MessageFormat.format(Messages.Tips_startFailed, Messages.Button_decryptRemote), -1);
			}
		}
		listener.finish(startRemoteDecrypt);
	}
	
	/**
	 * @Description: 先加载开标一览表结构
	 * @author guoyr 
	 */
	private void loadBidPrice(){
		// 开标解密时先判断是否已经存在报价结构
		if(bidPriceService.getBidPriceTableCount(tenderProjectEntity.getId()) == 0){
			// 如果不存在报价结构，则从招标文件中读取结构
			List<BidPrice> bidPriceTable = BidPriceUtils.getBidPriceTable(tenderProjectEntity.getId(), tenderProjectEntity.getTenderNo());
			if(null != bidPriceTable && bidPriceTable.size() > 0){
				bidPriceService.batchSaveResolverBidPriceTable(bidPriceTable);
			}
		}
	}
	
}
