package com.gx.obe.opening.thread;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.gx.obe.WebserviceConstants;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.common.widget.thread.MyThread;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.opening.listener.BatchStopRemoteDecryptListener;
import com.gx.obe.web.entity.TransactionServer;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.TenderProjectService;
import com.gx.utils.ActionLogUtils;

public class StopRemoteBidOpeningThread extends MyThread{
	
	private TenderProjectEntity tenderProjectEntity;
	private List<TenderProjectEntity> tenderPackList;
	private TransactionServer transactionServer;
	private BatchStopRemoteDecryptListener listener;
	private String bidopeningStage;
	private TenderProjectService tenderProjectService ;

	public StopRemoteBidOpeningThread(TenderProjectEntity tenderProjectEntity, List<TenderProjectEntity> tenderPackList, String bidopeningStage, TransactionServer transactionServer, BatchStopRemoteDecryptListener listener){
		this.tenderProjectEntity = tenderProjectEntity;
		this.tenderPackList = tenderPackList;
		this.bidopeningStage = bidopeningStage;
		this.transactionServer = transactionServer;
		this.listener = listener;
		tenderProjectService = new TenderProjectService();
	}
	
	public void run() {
		
		HttpUrl httpUrl = HttpUtils.getProjectHttpUrl(tenderProjectEntity, transactionServer, PropertiesUtils.getWebserviceProperty(WebserviceConstants.APPLY_STOP_DECRYPT, "bidOpeningHS.do?applyStopDecrypt"));
		if(null == httpUrl){
			return;
		}
		String openStageType = "00";
		boolean isPriceStage = false;
		boolean isBusAndTecStage = false;
		if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
			isPriceStage = true;
			openStageType = "02";
		}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
			isBusAndTecStage = true;
			openStageType = "01";
		}
		httpUrl.setAttribute("openStageType", openStageType);
		StringBuilder stageIds = new StringBuilder();
		Map<String, TenderProjectEntity> tenderPackMap = new HashMap<String, TenderProjectEntity>();
		
		if(null == tenderPackList){
			tenderPackList = new ArrayList<TenderProjectEntity>();
			tenderPackList.add(tenderProjectEntity);
		}
		for(TenderProjectEntity tenderPack : tenderPackList){
			if(stageIds.length() != 0){
				stageIds.append(",");
			}
			stageIds.append(tenderPack.getStageId());
			tenderPackMap.put(tenderPack.getTenderNo(), tenderPack);
		}
		httpUrl.setAttribute("stageIds", stageIds.toString());
		httpUrl.setName("停止远程解密");
		boolean stopFlag = HttpUtils.doPost(httpUrl).getStatus();
		ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"停止远程解密", httpUrl, stopFlag);
		if(stopFlag){
			for(TenderProjectEntity tenderPack : tenderPackList){
				if(isPriceStage){
					tenderPack.setPriceBidStatus(CommonEnum.STOP);
				}else if(isBusAndTecStage){
					tenderPack.setBusinessBidStatus(CommonEnum.STOP);
					tenderPack.setTechnologyBidStatus(CommonEnum.STOP);
				}else {
					tenderPack.setBusinessBidStatus(CommonEnum.STOP);
					tenderPack.setTechnologyBidStatus(CommonEnum.STOP);
					tenderPack.setPriceBidStatus(CommonEnum.STOP);
				}
			}
			httpUrl.setAttribute("stageIds", stageIds.toString());
			String[] updatePatams = null;
			if(isPriceStage){
				updatePatams = new String[]{"id", "priceBidStatus"};
			}else if(isBusAndTecStage){
				updatePatams = new String[]{"id","businessBidStatus", "technologyBidStatus"};
			}else {
				updatePatams = new String[]{"id","businessBidStatus", "technologyBidStatus", "priceBidStatus"};
			}
			tenderProjectService.batchUpdateProperty(tenderPackList, updatePatams);
		}
		if(null != listener){
			listener.stopRemoteDecrypt(stopFlag, stopFlag ? "已停止远程解密！": "停止远解密服务失败！");
		}
	}
}
