package com.gx.obe.opening.thread;

import java.text.MessageFormat;

import org.apache.log4j.Logger;
import org.dom4j.Document;

import com.gx.obe.WebserviceConstants;
import com.gx.obe.business.http.HttpResponse;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.common.widget.listener.ProgressMessageListener;
import com.gx.obe.common.widget.thread.MyRunnable;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.message.Messages;
import com.gx.obe.opening.listener.BatchStopRemoteDecryptListener;
import com.gx.obe.opening.webservice.SupplierBidOpeningResulterCreater;
import com.gx.obe.web.entity.TransactionServer;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.TenderProjectService;
import com.gx.obe.web.vo.TenderPackOpeningInfo;
import com.gx.utils.ActionLogUtils;

public class UploadBidOpeningResultThread extends MyRunnable{
	
	private TenderProjectEntity tenderProjectEntity;
	private ProgressMessageListener callBack;
	private Thread thread;
	private String bidopeningStage;
	private String publicType;
	private Logger LOG = Logger.getLogger(UploadBidOpeningResultThread.class);
	private TenderProjectService tenderProjectService;

	public UploadBidOpeningResultThread(TenderProjectEntity tenderProjectEntity, String bidopeningStage, String publicType, ProgressMessageListener callBack){
		this.tenderProjectEntity = tenderProjectEntity;
		this.bidopeningStage = bidopeningStage;
		this.publicType = publicType;
		this.callBack = callBack;
		tenderProjectService = new TenderProjectService();
		
	}
	public void start(){
		thread = new Thread(this);
		thread.start();
	}
	public void stop(){
		thread = null;
	}
	public void join(){
		if(null != thread){
			try {
				thread.join();
			} catch (InterruptedException e) {
			}
		}
	}
	public void run(){
		
		TransactionServer transactionServer = HttpUtils.getTransactionServer(tenderProjectEntity.getTransactionServerId());
		// 是否需要在完成开标前先停止解密
		final boolean needStopOpeningFirst = PropertiesUtils.getProperty(ConfigConstants.NEED_STOP_BIDOPENING_FIRST, "00").equals("01");
		if(needStopOpeningFirst){
			TenderPackOpeningInfo tenderPackOpeningInfo = tenderProjectService.getTenderPackOpeningInfo(tenderProjectEntity.getId(), bidopeningStage);
			if(!tenderPackOpeningInfo.isDecComplate() || tenderPackOpeningInfo.getDecCount() < tenderPackOpeningInfo.getSignCount()){
				
				// 先停止远程解密
				StopRemoteBidOpeningThread stopRemoteBidOpeningThread = new StopRemoteBidOpeningThread(tenderProjectEntity, null, bidopeningStage, transactionServer, new BatchStopRemoteDecryptListener(){
					public void stopRemoteDecrypt(boolean stopFlag,	String message) {
						if(!stopFlag){
							LOG.error(message);
						}
					}
				});
				stopRemoteBidOpeningThread.start();
				stopRemoteBidOpeningThread.join();
				
				RemoteDecryptStatusThread getRemoteDecryptStatusThread = new RemoteDecryptStatusThread(tenderProjectEntity, bidopeningStage, null, true);
				getRemoteDecryptStatusThread.start();
				getRemoteDecryptStatusThread.join();
			}
		}
		
		
		HttpUrl httpUrl = HttpUtils.getProjectHttpUrl(tenderProjectEntity, transactionServer, PropertiesUtils.getWebserviceProperty(WebserviceConstants.UPLOAD_SING_BIDRESULT_METHOD, ""));
		httpUrl.setName("上传开标结果");
		String openStageType = "00";
		if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
			openStageType = "02";
		}else if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_BUS_TEC)){
			openStageType = "01";
		}
		httpUrl.setAttribute("openStageType", openStageType);
		// 公开类型：默认或01为公开，02：不公开
		httpUrl.setAttribute("publicType", publicType);
		String bidOpeningResult = "";
		Document document = SupplierBidOpeningResulterCreater.createSupplierBidOpeningResultXml(tenderProjectEntity, bidopeningStage, false);
		if(null != document){
			bidOpeningResult = document.asXML();
			httpUrl.setAttribute("bidOpeningResult", bidOpeningResult);
		}
		HttpResponse httpResponse = HttpUtils.doPost(httpUrl);
		boolean uploadFlag = httpResponse.getStatus();
		ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"上传开标结果", httpUrl, uploadFlag);
		if(!uploadFlag) {
			httpResponse = HttpUtils.doPost(httpUrl);
			uploadFlag = httpResponse.getStatus();
			ActionLogUtils.addServerLog(tenderProjectEntity.getId(),"第二次上传开标结果", httpUrl, uploadFlag);
		}
		
		if(uploadFlag) {
			if(null != bidopeningStage && bidopeningStage.equals(TenderProjectEnum.STAGE_PRICE)){
				tenderProjectEntity.setUploadPriceOpeningResultStatus(CommonEnum.COMPLETE);
				tenderProjectService.updateAssignProperty(tenderProjectEntity, new String[]{"id","uploadPriceOpeningResultStatus"});
			}else {
				tenderProjectEntity.setUploadOpeningResultStatus(CommonEnum.COMPLETE);
				tenderProjectService.updateAssignProperty(tenderProjectEntity, new String[]{"id","uploadOpeningResultStatus"});
			}
			tenderProjectService.updateBidOpeningStatus(tenderProjectEntity, bidopeningStage, CommonEnum.COMPLETE);
		
			callBack.message(MessageFormat.format(Messages.Tips_uploadSuccessFormat, Messages.BidOpeningResult), 1);
		}else {
			if(null != httpResponse.getMsg())
			callBack.message(httpResponse.getMsg(), -1);
			
			callBack.message(MessageFormat.format(Messages.Tips_uploadFailFormat, Messages.BidOpeningResult), 1);
		}
		callBack.finish(uploadFlag);
	
	}
}
